package control;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Filtro que se encarga de decidir qu componente de control se debe ejecutar en funcin de 
 * la pgina JSP que est siendo visitada. La clase implementa la interfaz <code>Filter</code>, 
 * y como tal, se ejecutar antes de que la peticin del cliente llegue al Servlet destino 
 * (real o JSP compilado a Servlet). La clase implementa los tres mtodos que tiene la interfaz 
 * <code>Filter</code>. El mtodo <code>doFilter</code> que se ejecuta con cada peticin a 
 * cualquier pgina del proyecto como est configurado en "web.xml", es el mtodo de control de 
 * la Aplicacin Web, encargado de decidir qu clase contiene la lgica asociada a una pgina 
 * JSP determinada. 
 * 
 * @author 	David Domnguez
 * @see		javax.servlet.Filter
 */
public class ControlFilter implements Filter {

	/**
	 * Objeto de configuracin del Filtro.
	 */
	protected FilterConfig config = null;
	
	/**
	 * Mtodo que arranca y configura el filtro. Es llamado slo una vez cuando se carga el 
	 * filtro para usarlo con la aplicacin web. Simplemente le da el valor inicial al objeto 
	 * <code>FilterConfig</code>.
	 * 
	 * @param	filterConfig	Objeto con la configuracin y datos de inters del filtro
	 * @see 	javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig filterConfig) {
		config = filterConfig;
	}

	/**
	 * Mtodo centro de control de la Aplicacin, que se encarga de decidir qu componente est 
	 * asociado a cada pgina JSP visitada (si lo tiene). Se ejecuta siempre que se haga una 
	 * peticin al servidor, antes de que sta llegue al Sevlet destino (o pgina JSP compilada 
	 * a Servlet). Este filtro busca una clase java que se llame igual que la direccin pedida 
	 * por el cliente (sin extensin) y con la primera letra en maysculas, que implemente la 
	 * interfaz <code>Control</code>. Si la encuentra crea una instancia y ejecuta el mtodo 
	 * <code>doLogic</code>, encargado de manejar la lgica de negocio asociada a la pgina 
	 * visitada. Si <code>doLogic</code> devuelve true se siguen ejecutando los dems filtros 
	 * (si los hay) en la cadena hasta llegar al Servlet. Este mtodo es el control del 
	 * proyecto, encargado de buscar la lgica a ejecutar asociada a cada pgina requerida. Al 
	 * final, si todo ha ido bien, se llama al siguiente filtro en la cadena pasndole la 
	 * peticin y la respuesta.
	 * 
	 * Si la direccin introducida no contiene un nombre con extensin se busca ejecutar el 
	 * mtodo <code>doLogic</code> de una clase por defecto, "Default", que se puede definir 
	 * con un comportamiento por defecto para este tipo de direcciones.
	 * 
	 * @param	req					peticin del cliente. El mtodo espera que sea una 
	 * 								peticin http; en caso contrario lanza una excepcin de 
	 * 								servlet
	 * @param	res					respuesta a enviar al cliente
	 * @param	chain				representa la cadena de filtros que se aplica a la 
	 * 								peticin y a la respuesta actuales
	 * @throws	IOException			si hay algn problema leyendo o escribiendo en disco. La 
	 * 								lanza el mtodo "doLogic" de la clase que implementa la 
	 * 								interfaz "control" que se ejecute
	 * @throws	ServletException	si el mtodo "doLogic" de la clase que implementa la 
	 * 								interfaz "control" que se ejecute la lanza, o si la 
	 * 								peticin recibida no es http, o si hay alguna excepcin 
	 * 								creando la instancia adecuada de "control", que se lanza en 
	 * 								una excepcin de este tipo
	 * @see 	javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 * @see 	Control#doLogic(HttpServletRequest, HttpServletResponse)
	 */
	public void doFilter ( ServletRequest req, ServletResponse res,
			FilterChain chain) throws IOException, ServletException {
		
		// La peticin debe ser de tipo HTTP para poder acceder a los mtodos particulares de 
		// sta
		if (!(req instanceof HttpServletRequest)) {
			throw new ServletException("El Filtro requiere una peticin HTTP.");
		}
		
		// Determina el nombre del compontente de control implcito, que ser igual que el 
		// nombre de la pgina sin extensin y con la primera letra en maysculas
		HttpServletRequest request = (HttpServletRequest)req;
		HttpServletResponse response = (HttpServletResponse)res;
		String uri = request.getRequestURI();
		int inicio = uri.lastIndexOf("/")+1;
		int fin = uri.lastIndexOf(".");
		String nombre = "default";			// Nombre por defecto si la uri no es adecuada 
		if (inicio < fin) {
			nombre = uri.substring(inicio, fin);
		}
		if (Character.isLowerCase(nombre.charAt(0))){	// Pone la primera letra en maysculas
			nombre = Character.toUpperCase(nombre.charAt(0))+ nombre.substring(1);
		}
		boolean doFilter = true;		// Para seguir ejecutando la cadena de filtros si nada 
										// va mal
		// intenta cargar y ejecutar un componente de control implcito.
		try {
			// Busca en el paquete control la clase asociada
			Object o = Class.forName("control." + nombre).newInstance();
			// Debe implementar la interfaz "Control"
			if (!(o instanceof Control)) {
				throw new ServletException("Clase control."+ nombre +
						" no implementa control.Control");
			}
			Control control = (Control)o;
			doFilter = control.doLogic(request, response);	// Ejecuta la lgica asociada
		}
		catch (ClassNotFoundException e) {
			// Si la pgina jsp no tiene un componente de control implcito no hace nada. 
			// Sigue adelante. Simplemente indica que no necesita la preparacin lgica de los 
			// datos que se presentan en la pgina 
		}
		catch (InstantiationException e) {
			throw new ServletException(e);
		}
		catch (IllegalAccessException e) {
			throw new ServletException(e);
		}
		// sigue ejecutando los filtros subsiguientes si todo ha ido bien
		if (doFilter) {
			chain.doFilter(request, response);
		}
	}

	/**
	 * Se llama cuando el filtro se descarga de la memoria, tpicamente cuando la Aplicacin 
	 * se descarga. En este caso no es necesario hacer nada al descargar el filtro.
	 * 
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		// vaco
	}
}