package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import xml.test.*;

import utilidades.*;

/**
 * Componente de control implcito asociado a la pgina JSP "creartest.jsp". Es la lgica 
 * asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea 
 * llamada la pgina. Su funcin es comprobar que todos los parmetros iniciales de la toma de 
 * datos para crear un nuevo test sean correctos, en cuyo caso redirecciona la respuesta 
 * a la pgina de seleccionar los tems a incluir en el test. La clase implementa la interfaz 
 * <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el nico que 
 * contiene, y que es el encargado de realizar la lgica asociada a la pgina "creartest.jsp".
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Creartest implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina "creartest.jsp". 
	 * Ser llamado por el filtro <code>ControlFilter</code> en el caso de que detecte que la 
	 * pgina requerida es "creartest.jsp". Ya que es llamado desde un filtro, ser ejecutado 
	 * siempre que llegue una peticin de un cliente y antes de que la peticin llegue al 
	 * Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. Comprueba 
	 * uno por uno todos los parmetros, y, en caso de que sean correctos, crea un objeto <code>
	 * AssessmentTest</code> para guardar los datos asociados al test y lo guarda en el mbito 
	 * de sesin para utilizarlos ms adelante para terminar de crear el test.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see control.Control#doLogic(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 * @see 	control.ControlFilter
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EnteroPositivo
	 * @see		xml.test.AssessmentTest
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Comprueba si es la primera vez que se visita la pgina
		if (request.getParameter("Form") != null){
			
			// Caracteres reservados no permitidos
			final String reservados = ";/?:@&=+$,\\*\"<>|";

			String identificador = request.getParameter("Identif");	// Identificador del test 
			String titulo = request.getParameter("Tit");			// Ttulo dele test
			String instrucciones = request.getParameter("Instruc");	// Instrucciones del test
			// Permitir revisin del test al finalizarlo
			String rev = request.getParameter("allowReview");
			String selec = request.getParameter("Select");	// Nmero de tems a mostrar
			String Shuff = request.getParameter("Shuffle");		// Mezclar los tems o no
			
			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			boolean shuffle = false;		// Valor por defecto del parmetro
			if ( Shuff != null){			// Interpreta el valor marcado en la pgina 
				if (Shuff.equalsIgnoreCase("ON"))
					shuffle = true;
			}

			// Comprueba que estn introducidos todos los atributos obligatorios
			if (identificador != null && !identificador.equals("") &&
					titulo != null && !titulo.equals("") &&
					rev != null && !rev.equals("")){

				// Comprueba que los caracteres del identificador estn permitidos
		        for(int i = 0; i < identificador.length() && estado.isEstado() == true; i++) {
		        	char c = identificador.charAt(i);	// Lee el siguiente carcter
		        	int marca = reservados.indexOf(c);
		        	if (marca >= 0 ){
						estado.aadeError("Error: El carcter '" + reservados.charAt(marca) + 
								"' en la posicin " + (i+1) + " del Identificador no est " +
								"permitido<br />", "Identif");
		        	}
		        }

		        // Permitir o no la revisin del documento
				boolean revision = Boolean.parseBoolean(rev);
				
				// Comprueba que el nmero de tems a seleccionar est vaco o sea un valor
				// vlido
				EnteroPositivo seleccion;
				if (selec == null || selec.equals("")){
					seleccion = new EnteroPositivo("0");
					seleccion.esPositivo();			// Para actualizar el valor entero
				}
				else{
					seleccion = new EnteroPositivo(selec);
					MensajeEstado msg = seleccion.esPositivo();
					if(!msg.isEstado()){  // Si no es un nmero vlido y mayor o igual que cero
						estado.aadeError("Error: el nmero de tems a seleccionar " + 
								msg.getMensaje() + "<br />", "Select");
					}
				}
				
				// Comprueba que las instrucciones sean un cdigo XHTML vlido
				if (instrucciones != null && !instrucciones.equals("")){
					MensajeEstado msg = ParserNeutro.compruebaInstruccionesXHTML(instrucciones);
					if (msg.isEstado() == false)
						estado.aadeError(msg.getMensaje(), "Instruc");
				}

				// Si el estado es todo correcto pasa a generar el objeto contenedor del test 
				if ( estado.isEstado()){
					// Mtodo de ordenacin
					Ordering orden = null;
					if (shuffle == true)
						orden = new Ordering (shuffle);
					// Mtodo de seleccionar tems
					Selection select = null;
					// Si se selecciona algn nmero de tems del total, es sin reemplazamiento, 
					// ya que los tems son constantes, no contienen variables, y no tiene 
					// sentido repetirlos en una seccin
					if (seleccion.getEnteroPositivo() != 0)
						select = new Selection(seleccion.getEnteroPositivo(), false);
					// Se inicializa el objeto Seccin del test excepto las referencias a los
					// tems, con un identificador vlido que es vlido, por eso no se comprueba
					AssessmentSection seccion = 
						new AssessmentSection(new Identificador("seccion1"), titulo, true, 
								select, orden, instrucciones);
					// Objeto de control para permitir la revisin o no
					ItemSessionControl control = new ItemSessionControl(revision);
					// Se inicializa el objeto Parte de test con la Seccin anterior y con un 
					// identificador vlido, que como slo hay una parte, no se realiza ninguna 
					// comprobacin, el objeto control anterior, y estableciendo la navegacin y 
					// la presentacin a esos valores que son los nicos que soporta esta 
					// implementacin
					TestPart parte = new TestPart(new Identificador("parte1"), "nonlinear", 
							"simultaneous", control, seccion);
					// Objeto Test inicializado con los datos proporcionados por el usuario 
					// y los objetos creados ms arriba
					AssessmentTest test = new AssessmentTest(identificador, titulo, parte);
					// Guarda el objeto AsessmentTest en sesin para seguir utilizndolo luego
					HttpSession sesion = request.getSession();
					sesion.setAttribute("Test1", test);
					// Se borra de sesin el objeto con los tems a aadir para empezar de cero
					// aadiendo los tems en la pgina siguiente
					sesion.removeAttribute("Items_A_Aadir");
					// Redirecciona a la pgina de seleccionar los tems a incluir en el Test, 
					// pasando por la pgina de seleccionar la asignatura primero
					response.sendRedirect("seleccionasignatura.jsp?Destino=" +
							"seleccionaritems.jsp");
				}
			}
			// Si no estn todos los atributos obligatorios comprueba los que fallan
			// para informar al usuario
			else{
				if (identificador.equals(""))
					estado.aadeError("Debe introducir un Identificador<br />", "Identif");
				if (titulo.equals(""))
					estado.aadeError("Debe introducir un Ttulo<br />", "Tit");
				if (rev == null || rev.equals(""))
					estado.aadeError("Debe marcar si desea permitir la revisin del test " +
							"al finalizarlo<br />", "Identif");
			}
			
			// Si ha habido algn fallo pone los datos que ya ha introducido el usuario junto 
			// a los mensajes de aviso y el foco en el primer fallo encontrado
			if (estado.isEstado()== false){
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Focus", estado.getFocusON());
				request.setAttribute("Identif", identificador);
				request.setAttribute("Tit", titulo);
				request.setAttribute("Instruc", instrucciones);
				request.setAttribute("allowReview", rev);
				request.setAttribute("Select", selec);
				request.setAttribute("Shuffle", new Boolean(shuffle));
			}
		} else {
			// Si es la primera vez que se visita la pgina borra el objeto AssessmentTest del 
			// mbito de sesin para iniciar desde cero el test y pone el foco en el primer 
			// campo del formulario
			HttpSession sesion = request.getSession();
			sesion.removeAttribute("Test");
			request.setAttribute("Focus", "Identif");
		}		
		
		return true;
	}
}