package control;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import utilidades.ListaArchivos;
import utilidades.UrlUtils;

/**
 * Componente de control implcito asociado a la pgina JSP "explorador.jsp". Es la lgica 
 * asociada a esa pgina JSP. Su mtodo <code>doLogic</code> se ejecutar siempre que sea 
 * llamada la pgina. La pgina es el frame de la mitad izquierda de la pgina 
 * "buscadorimagenes.jsp", y es el explorador de imgenes y directorios para seleccionar una 
 * imagen. Se encarga de la navegacin por los directorios de imgenes, mostrando los 
 * subdirectorios y las imgenes que se encuentran en el directorio actual, permitiendo entrar 
 * en subdirectorios y volver atrs, previsualizando a su vez las imgenes en el frame derecho 
 * de la pgina para elegir una.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Explorador implements Control {

	/**
	 * Lgica control asociada a la pgina "explorador.jsp". Se ejecuta siempre que haya 
	 * alguna peticin con destino esa pgina. Es el frame izquierdo de la pgina 
	 * "buscadorimagenes.jsp". Ya que es llamado desde un filtro, ser ejecutado siempre que 
	 * llegue una peticin de un cliente y antes de que la peticin llegue al Servlet (pgina 
	 * JSP compilada a Servlet) que sirve la respuesta al cliente. Se encarga de mostrar al 
	 * usuario todos los directorios que se encuentren por debajo del de las Imgenes por 
	 * defecto (nunca subir ms arriba), permitindole ver las imgenes de los tipos que se 
	 * encuentren definidos en el mtodo <code>ListaImagenes</code> en esos directorios en el 
	 * frame derecho ("imagen.html") y navegar por el rbol de subdirectorios por debajo del 
	 * directorio inicial de imgenes, sin permitir subir ms arriba de ste. 
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se le enve la respuesta correctamente
	 * @throws	IOException			si hay algn problema en el acceso a las imgenes o los 
	 * 								directorios
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see		utilidades.ListaArchivos
	 * @see		utilidades.UrlUtils#encodeURLReserved(String)
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Contexto del Servlet
		ServletContext sc =	request.getSession().getServletContext();
		// Directorio a mostrar en este momento
		String dirImagenName = request.getParameter("DirectorioActual");
		// Directorio base de trabajo (guardado en "web.xml")
		String dirTrabajo = sc.getInitParameter("Directorio Trabajo");
		// Directorio de asignatura si venimos de la pgina de seleccin de asignatura
		String dirAsignatura = request.getParameter("DirectorioAsignatura");
		// Directorio base de las imgenes. El parmetro est definido en "web.xml"
		String dirInicial = sc.getInitParameter("Directorio Imgenes");
		// Directorio padre del actual (null si estamos en el base)
		String dirSuperior = null;
		
		if (dirImagenName == null){
			// Es la primera vez que entra en la pgina, as que va al directorio de imgenes 
			// inicial
			dirImagenName = "/" + dirAsignatura + dirInicial;
		}
		
		// Busca la segunda localizacin del carcter '/' para ver si estamos en el inicio
		int loc = dirImagenName.indexOf('/', 1);
		// Si longitud de la direccin actual es distinta de la que hay del inicio al segundo 
		// carcter '/' ms la del directorio inicial de los tems no estamos en el base, y por
		// lo tanto mostramos la opcin de ir al directorio superior
		if (loc + dirInicial.length() != dirImagenName.length())
			// Si no estamos en el inicial guarda el nombre del directorio padre para volver 
			// atrs
			dirSuperior = dirImagenName.substring(0, dirImagenName.lastIndexOf('/'));
		
		// Guarda en request los nombres del directorio actual y el del directorio padre
		request.setAttribute("DirectorioActual", dirImagenName);
		request.setAttribute("DirectorioSuperior", dirSuperior);
		
		// Comprueba que existe el directorio base de las imgenes
		File dirImagenes = new File(sc.getRealPath(dirTrabajo) + "/" + dirImagenName);
		if (!dirImagenes.exists())
			if (!dirImagenes.mkdir())
				throw new IOException("Error: No se pudo crear el directorio base de las " +
						"Imgenes");

		// Lista los nombres de los subdirectorios del directorio y lo guarda en request
		Vector /* String */ dirNames = ListaArchivos.ListaDirectorios(dirImagenes);
		request.setAttribute("NombresDirectorios",dirNames);

		// Lista los nombres de las imgenes del directorio y lo guarda en request
		Vector /* String */ imagNames = ListaArchivos.ListaImagenes(dirImagenes);
		// Guarda una versin de los nombres de los archivos de imgenes con todos los 
		// caracteres ilegales codificados
		Vector /* String */ imagNamesCodif = new Vector();
		for (int i = 0; i < imagNames.size(); i++){
			imagNamesCodif.add(UrlUtils.encodeURLReserved((String)imagNames.get(i)));
		}
		// URL absoluta del directorio de las imgenes ya codificado
		String dirURL;
		// string con la url que ha realizado el request
		String urlS = request.getRequestURL().toString();
		// url sin el nombre de la pgina actual, hasta el ltimo carcter '/'
		URL url = new URL(urlS.substring(0, urlS.lastIndexOf('/')));
		// Directorio de las imgenes desde el directorio base del proyecto
		String dir = "/" + dirTrabajo + dirImagenName + "/";
		// Direccin url absoluta del directorio a mostrar
		String urlAbs = url.toString() + dir;
		// Se codifican los caracteres ilegales de la url (excepto los reservados como los 
		// separadores, que se dejan igual)
		dirURL = UrlUtils.encodeURL(urlAbs);
		// Se guarda la URL codificada del directorio de las imgenes y los nombres de las
		// imgenes codificadas y sin codificar
		request.setAttribute("NombresImagenes", imagNames);
		request.setAttribute("NombresImagenesCodif", imagNamesCodif);
		request.setAttribute("URLImagenes", dirURL);
		
		return true;
	}
}