package control;

import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.commons.fileupload.servlet.*;
import org.apache.commons.fileupload.*;
import org.apache.commons.fileupload.disk.*;
import org.apache.commons.io.FilenameUtils;

import utilidades.ListaArchivos;

/**
 * Componente de control implcito asociado a la pgina JSP "fileupload.jsp". Es la lgica 
 * asociada a esa pgina JSP. Su mtodo <code>doLogic</code> se ejecutar siempre que sea 
 * llamada la pgina. Su funcin es la de permitir al usuario seleccionar un directorio en el 
 * servidor al que subir las imgenes seleccionadas en la pgina.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Fileupload implements Control {

	/**
	 * Lgica de control asociada a la pgina "fileupload.jsp", que se encarga de subir un 
	 * archivo al servidor. Ser llamado por el filtro <code>ControlFilter</code> en el caso de 
	 * que detecte que la pgina requerida es "fileupload.jsp". Ya que es llamado desde un 
	 * filtro, ser ejecutado siempre que llegue una peticin de un cliente y antes de que la 
	 * peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al 
	 * cliente. Se encarga de listar los subdirectorios que hay en el directorio de imgenes y 
	 * permitir entrar en ellos y volver atrs. Puede recibir informacin del resultado del 
	 * intento de crear un nuevo directorio para informar en "fileupload.jsp". Una vez 
	 * seleccionado el directorio debe subir el archivo, comprobando que sea de imagen, elegido 
	 * por el usuario y almacenarlo en el susodicho directorio, avisando al usuario del xito o 
	 * fracaso de tal accin.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se le enve la respuesta correctamente
	 * @throws	IOException			si hay algn problema con el acceso a los directorios o 
	 * 								guardando en disco las imgenes
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.ListaArchivos
	 * @see		org.apache.commons.fileupload.servlet.ServletRequestContext
	 * @see		org.apache.commons.fileupload.servlet.ServletFileUpload
	 * @see		org.apache.commons.fileupload.disk.DiskFileItemFactory
	 * @see		org.apache.commons.io.FilenameUtils
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Contexto del Servlet
		ServletContext sc =	request.getSession().getServletContext();
		// Directorio base de trabajo (guardado en "web.xml")
		String dirTrabajo = sc.getInitParameter("Directorio Trabajo");
		// Directorio de asignatura si venimos de la pgina de seleccin de asignatura
		String dirAsignatura = request.getParameter("DirectorioAsignatura");
		// Directorio a mostrar en este momento
		String dirImagenName = request.getParameter("DirectorioActual");
		// Directorio inicial de las imgenes (guardado en "web.xml")
		String dirInicial = sc.getInitParameter("Directorio Imgenes");
		// Aviso recibido de otra pgina a mostrar en esta
		String aviso = request.getParameter("Aviso");
		// Informacion recibida de otra pgina a mostrar en esta
		String informacion = request.getParameter("Informacion");
		// Directorio superior del actual (null si estamos en el inicial)
		String dirSuperior = null;

		if (dirImagenName == null){
			// Es la primera vez que entra en la pgina, as que va al directorio inicial de 
			// imgenes
			dirImagenName = "/" + dirAsignatura + dirInicial;
		}

		// Busca la segunda localizacin del carcter '/' para ver si estamos en el inicio
		int loc = dirImagenName.indexOf('/', 1);
		// Si longitud de la direccin actual es distinta de la que hay del inicio al segundo 
		// carcter '/' ms la del directorio inicial de los tems no estamos en el base, y por
		// lo tanto mostramos la opcin de ir al directorio superior
		if (loc + dirInicial.length() != dirImagenName.length())
			// Si no estamos en el inicial guarda el nombre del directorio padre para volver 
			// atrs
			dirSuperior = dirImagenName.substring(0, dirImagenName.lastIndexOf('/'));
		
		// Guarda en request los nombres del directorio actual y el del directorio padre
		request.setAttribute("DirectorioActual", dirImagenName);
		request.setAttribute("DirectorioSuperior", dirSuperior);
		
		// Comprueba que existe el directorio base de las imgenes
		File dirImagenes = new File(sc.getRealPath(dirTrabajo) + "/" + dirImagenName);
		if (!dirImagenes.exists())
			if (!dirImagenes.mkdir())
				throw new IOException("Error: No se pudo crear el directorio base de las " +
						"Imgenes");

		// Busca los directorios del directorio actual
		Vector /* String */ dirNames = ListaArchivos.ListaDirectorios(dirImagenes);
		request.setAttribute("NombresDirectorios",dirNames);

		// Comprobamos que tenemos una peticin de subida de archivo (peticin multipart)
		ServletRequestContext rqctxt = new  ServletRequestContext(request);
		boolean isMultipart = ServletFileUpload.isMultipartContent(rqctxt);
		
		if (isMultipart){
			// Crea un factory para archivos que guardar en disco, dejando por defecto el 
			// lmite de tamao para archivos en disco y el directorio temporal el del sistema
			DiskFileItemFactory factory = new DiskFileItemFactory();

			// Crea un manejador de archivos subidos
			ServletFileUpload upload = new ServletFileUpload(factory);

			try {
				// Crea un List con los archivos subidos (debe de ser slo uno)
				List /* FileItem */ items = upload.parseRequest(request);
			
				Iterator iter = items.iterator();
				while (iter.hasNext()) {
					FileItem fItem = (FileItem) iter.next();
					// Si el tem no es un campo de formulario (ser un archivo)
					if (!fItem.isFormField()) {
						String fName = FilenameUtils.getName(fItem.getName());
						// Comprueba que sea un archivo de imagen soportado
						String fExtension = FilenameUtils.getExtension(fName);
						if (fExtension.equalsIgnoreCase("bmp") || 
								fExtension.equalsIgnoreCase("gif") ||
								fExtension.equalsIgnoreCase("jpg") || 
								fExtension.equalsIgnoreCase("jpeg") ||
								fExtension.equalsIgnoreCase("png")){
							File uploadedFile = new File(dirImagenes,fName);
							// Comprueba que no exista ya el nombre de archivo
							if (!uploadedFile.exists()){
								fItem.write(uploadedFile);
								informacion = "Imagen subida con xito";
							}else{
								aviso = "Ese nombre de archivo ya existe";
							}
						}
						else{
							aviso = "Tipo de archivo incorrecto";
						}
					}
				}
			}
			catch (FileUploadException e){
				// Si la clase ServletFileUpload no ha podido procesar la peticin
				throw new ServletException("Error procesando la peticin", e);
			}
			catch (Exception e){
				// Si hay error al escribir el archivo o procesando la respuesta
				throw new ServletException ("Error guardando el archivo de imagen", e);
			}
		}

		// Avisos e informacin para el usuario de la pgina
		request.setAttribute("Aviso", aviso);
		request.setAttribute("Informacion", informacion);

		return true;
	}
}