package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import xml.items.GapmatchXML;
import utilidades.*;

/**
 * Componente de control implcito asociado a la pgina JSP "gapmatch.jsp". Es la lgica 
 * asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea 
 * llamada la pgina. Su funcin es comprobar que todos los parmetros iniciales de la toma de 
 * datos de una pregunta tipo "Gap Match" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la siguiente pgina de toma de datos de tipo "Gap Match". La clase implementa 
 * la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el 
 * nico que contiene, y que es el encargado de realizar la lgica asociada a la pgina 
 * "gapmatch.jsp".
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Gapmatch implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina "gapmatch.jsp". 
	 * Ser llamado por el filtro "ControlFilter" en el caso de que detecte que la pgina 
	 * requerida es "gapmatch.jsp". Ya que es llamado desde un filtro, ser ejecutado siempre 
	 * que llegue una peticin de un cliente y antes de que la peticin llegue al Servlet 
	 * (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. Comprueba uno por 
	 * uno todos los parmetros, y, en caso de que sean correctos, crea un objeto "GapmatchXML" 
	 * para guardar los datos asociados al tipo de pregunta y lo guarda en el mbito de sesin 
	 * para utilizarlos ms adelante para terminar de crear el tem.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EnteroPositivo
	 * @see		xml.items.GapmatchXML
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Comprueba que no sea la primera vez que se visita la pgina
		if (request.getParameter("Form") != null){

			String identificador = request.getParameter("Identif");	// Identificador del tem
			String titulo = request.getParameter("Tit");			// Ttulo del tem
			String instrucciones = request.getParameter("Instruc");	// Instrucciones del tem
			String pregunta = request.getParameter("Preg");			// Pregunta del tem
			String Shuff = request.getParameter("Shuffle");			// Mezclar respuestas o no
			String numOpc = request.getParameter("Num_opc");		// Nmero de opciones
			String numHuec = request.getParameter("Num_huec");		// Nmero de huecos

			// Caracteres reservados no permitidos
			final String reservados = ";/?:@&=+$,\\*\"<>|";

			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		

			boolean shuffle = false;	//valor por defecto
			if ( Shuff != null){		// Interpreta el valor marcado en la pgina
				if (Shuff.equalsIgnoreCase("ON"))
					shuffle = true;
			}

			// Comprueba que estn introducidos todos los atributos obligatorios
			if (identificador != null && !identificador.equals("") &&
					titulo != null && !titulo.equals("") &&
					numOpc != null && !numOpc.equals("") &&
					numHuec != null && !numHuec.equals("")){

				// Comprueba que los caracteres del identificador estn permitidos
		        for(int i = 0; i < identificador.length() && estado.isEstado() == true; i++) {
		        	char c = identificador.charAt(i);	// Lee el siguiente carcter
		        	int marca = reservados.indexOf(c);
		        	if (marca >= 0 ){
						estado.aadeError("Error: El carcter '" + reservados.charAt(marca) + 
								"' en la posicin " + (i+1) + " del Identificador no est " +
								"permitido<br />", "Identif");
		        	}
		        }

				// Comprueba que el nmero de opciones sea un valor vlido
				EnteroPositivo opciones = new EnteroPositivo(numOpc);
				MensajeEstado msg = opciones.esPositivo();
				if (msg.isEstado()){		// Si es un entero vlido y mayor que cero
					if (opciones.getEnteroPositivo() <= 1)		// Si es menor o igual que uno
						estado.aadeError("El nmero de opciones debe ser mayor que uno" +
								"<br />", "Num_opc");
				} else						// Si no es un entero positivo vlido
					estado.aadeError("Error: el nmero de opciones " + msg.getMensaje() + 
							"<br />", "Num_opc");
					
				// Comprueba que el nmero de huecos sea un valor vlido
				EnteroPositivo huecos = new EnteroPositivo(numHuec);
				msg = huecos.esPositivo();
				if (msg.isEstado()){		// Si es un entero vlido y mayor que cero
					if (huecos.getEnteroPositivo() < 1)			// Si es menor que uno
						estado.aadeError("El nmero de huecos debe ser mayor o igual que 1" +
								"<br />", "Num_huec");
				} else 						// Si no es un entero positivo vlido
					estado.aadeError("Error: el nmero de huecos es un " + msg.getMensaje() + 
							"<br />", "Num_huec");
				
				// Comprueba que las instrucciones sean un cdigo XHTML vlido
				if (instrucciones != null && !instrucciones.equals("")){
					msg = ParserNeutro.compruebaInstruccionesXHTML(instrucciones);
					if (msg.isEstado() == false)
						estado.aadeError(msg.getMensaje(), "Instruc");
				}

				// Si el estado es todo correcto pasa a generar el objeto contenedor del tem 
				if (estado.isEstado() == true){
					// Crea un objeto que contiene las caractersticas del tem Gap Match a 
					// crear
					GapmatchXML gmXML = new GapmatchXML(identificador, titulo, instrucciones, 
							pregunta, shuffle);
					// Guarda el objeto Gapmatch en sesin para seguir utilizndolo luego
					HttpSession sesion = request.getSession();
					sesion.setAttribute("GapMatch", gmXML);
					// Redireccionamos a la pgina de introducir las respuestas indicando el
					// nmero de opciones, el de huecos y si hay que mezclar
					response.sendRedirect("respuestasgapmatch.jsp?NumOpc=" + numOpc + 
							"&NumHuec=" + numHuec + "&Shuffle=" + shuffle);
				}
			}
			// si no estn todos los atributos obligatorios comprueba los que faltan
			// para informar al usuario
			else{
				if (identificador.equals(""))
					estado.aadeError("Debe introducir un Identificador<br />", "Identif");
				if (titulo.equals(""))
					estado.aadeError("Debe introducir un Ttulo<br />", "Tit");
				if (numOpc.equals(""))
					estado.aadeError("Debe introducir un nmero de opciones<br />", 
							"Num_opc");
				if (numHuec.equals(""))
					estado.aadeError("Debe introducir un nmero de huecos<br />", "Num_huec");
			}
			
			// Si ha habido algn fallo
			if (estado.isEstado()== false){
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Focus", estado.getFocusON());
				request.setAttribute("Identif", identificador);
				request.setAttribute("Tit", titulo);
				request.setAttribute("Instruc", instrucciones);
				request.setAttribute("Preg", pregunta);
				request.setAttribute("Num_opc", numOpc);
				request.setAttribute("Num_huec", numHuec);
				request.setAttribute("Shuffle", new Boolean(shuffle));
			}
		} else {
			// Si es la primera vez que se visita la pgina borra el objeto GapMatchXML del 
			// mbito de sesin para iniciar desde cero el tem y pone el foco en el primer 
			// campo del formulario
			HttpSession sesion = request.getSession();
			sesion.removeAttribute("GapMatch");
			request.setAttribute("Focus", "Identif");			
		}
		
		return true;
	}
}