package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import utilidades.*;

import xml.items.HottextXML;

/**
 * Componente de control implcito asociado a la pgina JSP "hottext.jsp". Es la lgica 
 * asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea 
 * llamada la pgina. Su funcin es comprobar que todos los parmetros iniciales de la toma de 
 * datos de una pregunta tipo "Hot Text" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la siguiente pgina de toma de datos de tipo "Hot Text". La clase implementa la 
 * interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el 
 * nico que contiene, y que es el encargado de realizar la lgica asociada a la pgina 
 * "hottext.jsp".
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Hottext implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina "hottext.jsp". 
	 * Ser llamado por el filtro <code>ControlFilter</code> en el caso de que detecte que la 
	 * pgina requerida es "hottext.jsp". Ya que es llamado desde un filtro, ser ejecutado 
	 * siempre que llegue una peticin de un cliente y antes de que la peticin llegue al 
	 * Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. Comprueba 
	 * uno por uno todos los parmetros, y, en caso de que sean correctos, crea un objeto 
	 * "HottextXML" para guardar los datos asociados al tipo de pregunta y lo guarda en el 
	 * mbito de sesin para utilizarlos ms adelante para terminar de crear el tem.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EnteroPositivo
	 * @see		xml.items.HottextXML
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Comprueba que no sea la primera vez que se visita la pgina
		if (request.getParameter("Form") != null){

			// Caracteres reservados no permitidos
			final String reservados = ";/?:@&=+$,\\*\"<>|";

			String identificador = request.getParameter("Identif");	// Identicador del tem
			String titulo = request.getParameter("Tit");			// Ttulo del tem
			String instrucciones = request.getParameter("Instruc");	// Instrucciones del tem
			String pregunta = request.getParameter("Preg");			// Pregunta del tem
			String numHot = request.getParameter("Num_hot");		// Nmero de hottexts

			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			// Comprueba que estn introducidos todos los atributos obligatorios
			if (identificador != null && !identificador.equals("") &&
					titulo != null && !titulo.equals("") &&
					numHot != null && !numHot.equals("")){

				// Comprueba que los caracteres del identificador estn permitidos
		        for(int i = 0; i < identificador.length() && estado.isEstado() == true; i++) {
		        	char c = identificador.charAt(i);	// Lee el siguiente carcter
		        	int marca = reservados.indexOf(c);
		        	if (marca >= 0 ){
						estado.aadeError("Error: El carcter '" + reservados.charAt(marca) + 
								"' en la posicin " + (i+1) + " del Identificador no est " +
								"permitido<br />", "Identif");
		        	}
		        }

				// Comprueba que el nmero de hottext sea un valor vlido
				EnteroPositivo numhottexts= new EnteroPositivo(numHot);
				MensajeEstado msg = numhottexts.esPositivo();
				if (msg.isEstado()){		// Si es un nmero vlido y mayor o igual que cero
					if (numhottexts.getEnteroPositivo() <= 1) 	// Si es menor o igual que uno
						estado.aadeError("El nmero de hottexts debe ser mayor que uno" +
								"<br />", "Num_hot");
				} else						// Si no es un entero positivo vlido
					estado.aadeError("Error: el nmero de hottext " + msg.getMensaje() + 
							"<br />", "Num_hot");
				
				// Comprueba que las instrucciones sean un cdigo XHTML vlido
				if (instrucciones != null && !instrucciones.equals("")){
					msg = ParserNeutro.compruebaInstruccionesXHTML(instrucciones);
					if (msg.isEstado() == false)
						estado.aadeError(msg.getMensaje(), "Instruc");
				}

				// Si todo es correcto
				if (estado.isEstado() == true){
					// Crea un objeto HottextXML con las caractersticas del tem Hot Text a 
					// crear
					HottextXML htXML = new HottextXML(identificador, titulo, instrucciones, 
							pregunta);
					// Guarda el objeto HottextXML en sesin para seguir utilizndolo luego
					HttpSession sesion = request.getSession();
					sesion.setAttribute("HotText", htXML);
					// Redirecciona a la pgina de introducir los hottexts y los textos 
					// indicando el nmero de hottexts
					response.sendRedirect("respuestashottext.jsp?NumHot=" + numHot);
				}
			}
			// Si no estn todos los atributos obligatorios comprueba los que fallan
			// para informar al usuario
			else{
				if (identificador.equals(""))
					estado.aadeError("Debe introducir un Identificador<br />", "Identif");
				if (titulo.equals(""))
					estado.aadeError("Debe introducir un Ttulo<br />", "Tit");
				if (numHot.equals(""))
					estado.aadeError("Debe introducir un nmero de hottexts<br />", 
							"Num_hot");
			}

			// Si ha habido algn fallo pone los datos que ya ha introducido el usuario junto 
			// a los mensajes de aviso y el foco en el primer fallo encontrado
			if (estado.isEstado()== false){
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Focus", estado.getFocusON());
				request.setAttribute("Identif", identificador);
				request.setAttribute("Tit", titulo);
				request.setAttribute("Instruc", instrucciones);
				request.setAttribute("Preg", pregunta);
				request.setAttribute("Num_hot", numHot);
			}
		} else {
			// Si es la primera vez que se visita la pgina borra el objeto hottextXML del 
			// mbito de sesin para iniciar desde cero el tem y pone el foco en el primer 
			// campo del formulario
			HttpSession sesion = request.getSession();
			sesion.removeAttribute("HotText");
			request.setAttribute("Focus", "Identif");
		}
		
		return true;
	}
}