package control;

import utilidades.*;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import xml.items.MatchXML;

/**
 * Componente de control implcito asociado a la pgina JSP "match.jsp". Es la lgica asociada 
 * a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea llamada la 
 * pgina. Su funcin es comprobar que todos los parmetros iniciales de la toma de datos de 
 * una pregunta tipo "Match" sean correctos, en cuyo caso redirecciona la respuesta a la 
 * siguiente pgina de toma de datos de tipo "Match". La clase implementa la interfaz <code>
 * Control</code>, implementando su mtodo <code>doLogic</code>, que es el nico que contiene, 
 * y que es el encargado de realizar la lgica asociada a la pgina "match.jsp".
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Match implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina "match.jsp". Ser 
	 * llamado por el filtro <code>ControlFilter</code> en el caso de que detecte que la 
	 * pgina requerida es "match.jsp". Ya que es llamado desde un filtro, ser ejecutado 
	 * siempre que llegue una peticin de un cliente y antes de que la peticin llegue al 
	 * Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. Comprueba 
	 * uno por uno todos los parmetros, y, en caso de que sean correctos, crea un objeto <code>
	 * MatchXML</code> para guardar los datos asociados al tipo de pregunta y lo guarda en el 
	 * mbito de sesin para utilizarlos ms adelante para terminar de crear el tem.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EnteroPositivo
	 * @see		xml.items.MatchXML
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Comprueba que no sea la primera vez que se entra en la pgina
		if (request.getParameter("Form") != null){

			// Caracteres reservados no permitidos
			final String reservados = ";/?:@&=+$,\\*\"<>|";

			String identificador = request.getParameter("Identif");	// Identificador del tem
			String titulo = request.getParameter("Tit");			// Ttulo del tem
			String instrucciones = request.getParameter("Instruc");	// Instrucciones del tem
			String pregunta = request.getParameter("Preg");			// Pregunta del tem
			// Nmero de elementos del grupo 1
			String grupo1 = request.getParameter("NumGrupo1");
			// Nmero de elementos del grupo 2
			String grupo2 = request.getParameter("NumGrupo2");
			// Mezclar las preguntas o no
			String Shuff = request.getParameter("Shuffle");
			
			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			boolean shuffle = false;			//valor por defecto
			if ( Shuff != null){				// Interpreta el valor marcado en la pgina
				if (Shuff.equalsIgnoreCase("ON"))
						shuffle = true;
			}

			// Comprueba que estn introducidos todos los atributos obligatorios
			if (identificador != null && !identificador.equals("") &&
				titulo != null && !titulo.equals("") &&
				grupo1 != null && !grupo1.equals("") &&
				grupo2 != null && !grupo2.equals("")){

				// Comprueba que los caracteres del identificador estn permitidos
		        for(int i = 0; i < identificador.length() && estado.isEstado() == true; i++) {
		        	char c = identificador.charAt(i);	// Lee el siguiente carcter
		        	int marca = reservados.indexOf(c);
		        	if (marca >= 0 ){
						estado.aadeError("Error: El carcter '" + reservados.charAt(marca) + 
								"' en la posicin " + (i+1) + " del Identificador no est " +
								"permitido<br />", "Identif");
		        	}
		        }

				// Recoge los valores del nmero de elementos en cada grupo
				EnteroPositivo numGrupo1 = new EnteroPositivo(grupo1);
				EnteroPositivo numGrupo2 = new EnteroPositivo(grupo2);
				// Comprueba el nmero de elementos del grupo 1
				MensajeEstado msg = numGrupo1.esPositivo();
				if(msg.isEstado()){		// Si es un nmero vlido y mayor o igual que cero
					if(numGrupo1.getEnteroPositivo() < 1)	// Si es menor o igual que uno
						estado.aadeError("Error: El nmero de elementos del grupo 1 debe " +
								"ser un nmero mayor que cero<br />", "NumGrupo1");
				} else 					// Si no es un entero positivo vlido
					estado.aadeError("Error: el nmero de elementos del grupo 1 " + 
							msg.getMensaje() + "<br />", "NumGrupo1");
				// Comprueba el nmero de elementos del grupo 2
				msg = numGrupo2.esPositivo();
				if(msg.isEstado()){		// Si es un nmero vlido y mayor o igual que cero
					if(numGrupo2.getEnteroPositivo() < 1)	// Si es menor o igual que uno
						estado.aadeError("Error: El nmero de elementos del grupo 2 debe " +
								"ser un nmero mayor que cero<br />", "NumGrupo2");
				} else 					// Si no es un entero positivo vlido
					estado.aadeError("Error: el nmero de elementos del grupo 2 " + 
							msg.getMensaje() + "<br />", "NumGrupo2");

				// Comprueba que las instrucciones sean un cdigo XHTML vlido
				if (instrucciones != null && !instrucciones.equals("")){
					msg = ParserNeutro.compruebaInstruccionesXHTML(instrucciones);
					if (msg.isEstado() == false)
						estado.aadeError(msg.getMensaje(), "Instruc");
				}

				// Si el estado es todo correcto pasa a generar el objeto contenedor del tem
				if (estado.isEstado()== true){
					// Crea un objeto que contiene las caractersticas del tem Match a crear
					MatchXML maXML = new MatchXML(identificador, titulo, instrucciones,
							pregunta, shuffle);
					// Guarda el objeto Match en sesin para seguir utilizndolo en la sesin
					HttpSession sesion = request.getSession();
					sesion.setAttribute("Match", maXML);
					// Redirecciona a la pgina de introducir las respuestas indicando los 
					// nmeros de elementos de cada grupo y si hay que mezclar
					response.sendRedirect("respuestasmatch.jsp?NumGrupo1=" + grupo1 + 
							"&NumGrupo2=" +	grupo2 + "&Shuffle=" + shuffle);
				}
			}
			// si no estn todos los atributos obligatorios comprueba los que fallan
			// para informar al usuario.
			else{
				if (identificador.equals(""))
					estado.aadeError("Debe introducir un Identificador<br />", "Identif");
				if (titulo.equals(""))
					estado.aadeError("Debe introducir un Ttulo<br />", "Tit");
				if (grupo1.equals(""))
					estado.aadeError("Debe introducir el nmero de elementos del grupo 1" +
							"<br />", "NumGrupo1");
				if (grupo2.equals(""))
					estado.aadeError("Debe introducir el nmero de elementos del grupo 2" +
							"<br />", "NumGrupo2");
			}
			// Si ha habido algn fallo pone los datos que ya ha introducido el usuario junto 
			// a los mensajes de aviso y el foco en el primer fallo encontrado
			if (estado.isEstado() == false){
				request.setAttribute("Identif", identificador);
				request.setAttribute("Tit", titulo);
				request.setAttribute("Instruc", instrucciones);
				request.setAttribute("Preg", pregunta);
				request.setAttribute("NumGrupo1",grupo1);
				request.setAttribute("NumGrupo2",grupo2);
				request.setAttribute("Shuffle",new Boolean(shuffle));
				request.setAttribute("Aviso",estado.getMensaje());
				request.setAttribute("Focus",estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina borra el objeto matchXML del 
			// mbito de sesin para iniciar desde cero el tem y pone el foco en el primer 
			// campo del formulario
			HttpSession sesion = request.getSession();
			sesion.removeAttribute("Match");
			request.setAttribute("Focus", "Identif");
		}

		return true;
	}
}