package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import xml.items.HottextXML;

import utilidades.*;

/**
 * Componente de control implcito asociado a la pgina JSP "respuestashottext.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Su funcin es comprobar que todos los parmetros finales de la toma 
 * de datos de una pregunta tipo "Hot Text" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la pgina "seleccionasignatura.jsp" para elegir donde guardar el tem. La clase 
 * implementa la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, 
 * que es el encargado de realizar la lgica asociada a la pgina "respuestashottext.jsp". 
 * Tiene otro mtodo que simplemente se ejecuta si los parmetros son correctos para 
 * redireccionar la respuesta y terminar de crear el objeto <code>HottextXML</code>.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Respuestashottext implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "respuestashottext.jsp". Ser llamado por el filtro <code>ControlFilter</code> en el 
	 * caso de que detecte que la pgina requerida es "respuestashottext.jsp". Ya que es 
	 * llamado desde un filtro, ser ejecutado siempre que llegue una peticin de un cliente y 
	 * antes de que la peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la 
	 * respuesta al cliente. Comprueba uno por uno todos los parmetros, y, en caso de que sean 
	 * correctos,  llama al mtodo <code>terminaHottext</code> para terminar de crear el objeto 
	 * del tem y redireccionar la respuesta.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see		javax.servlet.http.HttpSession
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.EnteroPositivo
	 * @see		utilidades.Identificador
	 * @see		utilidades.Comprobaciones
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Nombre base para los identificadores de los hottexts por defecto
		final String identificadorBase = "Hottext";
		request.setAttribute("IdentificadorBase", identificadorBase);
		
		// Este elemento siempre estar presente en request porque viene de la pgina anterior
		String numHottext= request.getParameter("NumHot");	// Nmero de hottexts
		int numHot = Integer.parseInt(numHottext);		// nmero de hottexts en entero
		// Array con los identificadores de los hottexts
		Identificador[] identHottexts = new Identificador[numHot];
			
		//Comprueba si es la primera vez que se entra en la pgina
		if(request.getParameter("Form") != null){
			
			String[] hottexts = new String [numHot];		// Array con los hottexts

			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			int i;
			// Lee los fragmentos de texto que acompaan a los huecos
			String[] fragmentosTexto = new String[numHot + 1];
			for (i = 0; i < (numHot + 1); i++)
				fragmentosTexto[i] = request.getParameter("Texto"+i);
			
			// Lee los hottexts introducidos, y si alguno est vaco aade el cdigo de error
			for(i = 0; i < numHot; i++){
				hottexts[i] = request.getParameter("Hottext"+i);
				if (hottexts[i] == null || hottexts[i].equals(""))
					estado.aadeError("El hottext " + (i+1)+ " no puede quedar vaco<br />", 
							"Hottext" + i);
			}
			
			// Lee los identificadores de los hottexts comprobando que sean correctos y no 
			// haya ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identHottexts, "IdentHot", estado, request, "");
			
			// Comprueba los checkbox de hottext correcto que al menos uno est marcado y suma 
			// el total de respuestas marcadas como correctas
			boolean[] correcta = new boolean[numHot];		// Respuestas correcta o no
			int numCorrectas = Comprobaciones.sumaCorrectas(correcta, "Correcto", request);
			if (numCorrectas == 0)	// Sin ningn checkbox de respuesta correcta est marcado
				estado.aadeError("Debe marcar al menos una opcin correcta<br />", 
						"Correcto0");
			
			// Lee el nmero mximo de elecciones introducido por el usuario
			String max = request.getParameter("Max");
			EnteroPositivo maxChoices = new EnteroPositivo(max);
			// Comprueba que el valor introducido sea un valor correcto, mayor que el nmero 
			// de respuestas correctas o cero, y menor que el nmero de respuestas posibles
			Comprobaciones.testMaximo(maxChoices, estado, "Max", numCorrectas, numHot);
			
			//Si todo est bien llama al mtodo que finaliza el tem y redirecciona al cliente
			if (estado.isEstado() == true){
				terminaGapmatch(request, response, fragmentosTexto, hottexts, identHottexts, 
						maxChoices, correcta);
			} else {
				// Si ha habido algn fallo pone los datos que ya ha introducido el usuario 
				// junto a los mensajes de aviso y el foco en el primer fallo encontrado
				request.setAttribute("Texto", fragmentosTexto);
				request.setAttribute("Hottexts", hottexts);
				request.setAttribute("IdentHot", identHottexts);
				request.setAttribute("Max", max);
				request.setAttribute("Correcto", correcta);
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Focus", estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina pone el foco en el primer campo
			request.setAttribute("Focus", "Max");
			// Se inicializa los identificadores a unos valores por defecto
			for (int i = 0; i < identHottexts.length; i++)
				identHottexts[i] = new Identificador(identificadorBase + (i+1));
			request.setAttribute("IdentHot", identHottexts);
		}
		
		// El nmero de hottexts hace falta siempre, sea la primera vez que visita la pgina o 
		// no y pone el foco en el primer campo del formulario
		request.setAttribute("NumHot",numHottext);

		return true;
	}
	/**
	 * Este mtodo se ejecuta una vez que se comprueba que todos los parmetros son correctos. 
	 * Usa el objeto <code>HottextXML</code> creado anteriormente y almacenado en sesin para 
	 * guardar el resto de los datos asociados a la pregunta, redireccionando la respuesta a 
	 * la pgina "seleccionasignatura.jsp" para que el usuario seleccione una asignatura en la 
	 * que guardar el tem XML. Si el objeto <code>HottextXML</code> no se ha podido recuperar 
	 * de sesin porque sta ha expirado, redirecciona la respuesta a una pgina para informar 
	 * al usuario.
	 * 
	 * @param 	req				peticin del cliente para acceder a la sesin
	 * @param 	res				respuesta para el cliente para redireccionarlo a otra pgina
	 * @param	textos			array con los textos que rodean a los hottexts
	 * @oaram	hott			array con los hottexts
	 * @param 	iHott			array de identificador con los identificadores de los hottexts
	 * @param 	maxOpc			nmero mximo de hottexts que se pueden seleccionar
	 * @param 	correct			array de boolean indicando si el hottext es correcto
	 * @throws 	IOException		si hay algn error al redireccionar la respuesta
	 * @see		xml.items.HottextXML
	 * @see		javax.servlet.http.HttpSession
	 * @see		javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
	 */
	private void terminaGapmatch(HttpServletRequest req, HttpServletResponse res, 
			String[] textos, String[] hott, Identificador[] iHott,EnteroPositivo max, 
			boolean[] correct) throws IOException{

		// Objeto sesin para acceder a los objetos ah almacenados		
		HttpSession sesion = req.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){	
			// Rellena los campos que faltan en el objeto ChoiceXML que se cre en la 
			// primera parte de la toma de datos
			HottextXML htXML = (HottextXML)sesion.getAttribute("HotText");
			htXML.setTextos(textos);
			htXML.setHottexts(hott);
			htXML.setIdentHottexts(iHott);
			htXML.setMaxChoices(max.getEnteroPositivo());
			htXML.setCorrecto(correct);
			// Almacena el tem modificado en sesin para utilizarlo ms adelante
			sesion.setAttribute("AItem", htXML);
			// Redirecciona la respuesta a "seleccionasignatura.jsp" indicndole la direccin 
			// a la que debe ir despus
			res.sendRedirect("seleccionasignatura.jsp?Destino=finitem.jsp");
		} else			// Si la sesin ha expirado
			res.sendRedirect("errorfinsesion.jsp");
	}

}