package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import xml.items.TextentryXML;

import utilidades.*;

/**
 * Componente de control implcito asociado a la pgina JSP "respuestastextentry.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Su funcin es comprobar que todos los parmetros finales de la toma 
 * de datos de una pregunta tipo "Text Entry" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la pgina "seleccionasignatura.jsp" para elegir donde guardar el tem. La clase 
 * implementa la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, 
 * que es el encargado de realizar la lgica asociada a la pgina "respuestastextentry.jsp". 
 * Tiene otro mtodo que simplemente se ejecuta si los parmetros son correctos para 
 * redireccionar la respuesta y terminar de crear el objeto <code>TextentryXML</code>.
 * 
 * 
 * @author 	David Domnguez
 * @see		Control
 * 
 */
public class Respuestastextentry implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "respuestashottext.jsp". Ser llamado por el filtro <code>ControlFilter</code> en el 
	 * caso de que detecte que la pgina requerida es "respuestastextentry.jsp". Ya que es 
	 * llamado desde un filtro, ser ejecutado siempre que llegue una peticin de un cliente y 
	 * antes de que la peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la 
	 * respuesta al cliente. Comprueba uno por uno todos los parmetros, y, en caso de que sean 
	 * correctos, llama al mtodo <code>terminaTextentry</code> para terminar de crear el 
	 * objeto del tem y redireccionar la respuesta. 
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EnteroPositivo
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Comprueba si es la primera vez que se entra en la pgina
		if(request.getParameter("Form") != null){
			
			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		

			// Recoge el valor de los dos campos de texto que pueden estar vacos
			String texto1 = request.getParameter("Texto1");
			String texto2 = request.getParameter("Texto2");
		
			// Comprueba que se haya introducido la respuesta correcta
			String respuesta = request.getParameter("Respuesta");		// Respuesta
			String longitud = request.getParameter("Longitud");			// Longitud esperada
			if (respuesta == null || respuesta.equals(""))
				estado.aadeError("La respuesta correcta no puede ser una cadena vaca<br />",
						"Respuesta");
			
			// Procesa la longitud esperada, comprobando que sea vlida si se ha introducido 
			// alguna. Si no se ha introducido ninguna no pasa nada, no es un campo obligatorio
			EnteroPositivo longEsperada = null;
			MensajeEstado msg = null;
			if (longitud != null && !longitud.equals("")){
				longEsperada = new EnteroPositivo(longitud);
				msg = longEsperada.esPositivo();
				if (!msg.isEstado())		// Si no es un entero positivo
					estado.aadeError("Error: la longitud esperada " + msg.getMensaje() + 
							"<br />", "Longitud");
				else if (longEsperada.getEnteroPositivo() < 1)
					// Si el nmero es un entero positivo correcto pero es menor que uno
					estado.aadeError("La longitud esperada debe ser mayor o igual a 1<br />", 
							"Longitud");
			}
			
			// Si todo ha ido bien vamos a generar el archivo xml
			if (estado.isEstado() == true){
				terminaTextentry(request, response, respuesta, longEsperada, texto1, texto2);
			} else {	
				// Si ha habido algn fallo pone los datos que ya ha introducido el usuario 
				// junto a los mensajes de aviso y el foco en el primer fallo encontrado
				request.setAttribute("Respuesta", respuesta);
				request.setAttribute("Longitud", longitud);
				request.setAttribute("Texto1", texto1);
				request.setAttribute("Texto2", texto2);
				request.setAttribute("Aviso",estado.getMensaje());
				request.setAttribute("Focus",estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina pone el foco en el primer elemento
			request.setAttribute("Focus","Texto1");
		}
		
		return true;
	}
	/**
	 * Este mtodo se ejecuta una vez que se comprueba que todos los parmetros son correctos. 
	 * Usa el objeto <code>TextentryXML</code> creado anteriormente y almacenado en sesin 
	 * para guardar el resto de los datos asociados a la pregunta, redireccionando la 
	 * respuesta a la pgina "seleccionasignatura.jsp" para que el usuario seleccione una 
	 * asignatura en la que guardar el tem XML. Si el objeto <code>ChoiceXML</code> no se ha 
	 * podido recuperar de sesin porque sta ha expirado, redirecciona la respuesta a una 
	 * pgina para informar al usuario.
	 * 
	 * @param 	req				peticin del cliente para acceder a la sesin
	 * @param 	res				respuesta para el cliente para redireccionarlo a otra pgina
	 * @param 	respuesta		string con la respuesta correcta
	 * @param 	longitud		enteroPositivo con la longitud esperada de la respuesta
	 * @param	texto1			Texto antes del hueco para introducir la respuesta
	 * @param	texto2			Texto despus del hueco para introducir la respuesta
	 * @throws 	IOException		si hay algn error al redireccionar la respuesta
	 * @see		xml.items.TextentryXML
	 * @see		javax.servlet.http.HttpSession
	 * @see		javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
	 */
	private void terminaTextentry(HttpServletRequest req, HttpServletResponse res, 
			String respuesta, EnteroPositivo longitud, String texto1, String texto2) 
			throws IOException{
		HttpSession sesion = req.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){	
			// Rellena los campos que faltan en el objeto ChoiceXML que se cre en la 
			// primera parte de la toma de datos
			TextentryXML teXML = (TextentryXML)sesion.getAttribute("TextEntry");
			teXML.setRespuesta(respuesta);
			if (longitud!= null)
				teXML.setLongitud(longitud.getEnteroPositivo());
			teXML.setTextos(texto1, texto2);
			// Almacena el tem modificado en sesin para utilizarlo ms adelante
			sesion.setAttribute("AItem", teXML);
			// Redirecciona la respuesta a "seleccionasignatura.jsp" indicndole la direccin 
			// a la que debe ir despus
			res.sendRedirect("seleccionasignatura.jsp?Destino=finitem.jsp");
		} else			// Si la sesin ha expirado
			res.sendRedirect("errorfinsesion.jsp");
	
	}
}