package control;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import utilidades.ListaArchivos;
import utilidades.ResumenItem;
import utilidades.MensajeEstado;

import xml.test.*;

/**
 * Componente de control implcito asociado a la pgina JSP "seleccionaritems.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Su funcin es mostrar al usuario los tems a seleccionar de los 
 * directorios de las asignaturas y permitirle elegir los que desea aadir al test, que se 
 * mostrarn en la capa de la derecha de la misma pgina. La clase implementa la interfaz 
 * <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el encargado de 
 * realizar la lgica asociada a la pgina "seleccionaritems.jsp". Contiene, adems, otros 
 * mtodos para ayudar en las tareas de aadir, eliminar y marcar a los tems ya seleccionados.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Seleccionaritems implements Control {

	/**
	 * Muestra el directorio de los tems de una asignatura permitiendo navegar por los 
	 * subdirectorios y elegir los tems a incluir en el test, que se mostrarn en otra capa de 
	 * la misma pgina. Ser llamado por el filtro <code>ControlFilter</code> en el caso de que 
	 * detecte que la pgina requerida es "seleccionaritems.jsp". Ya que es llamado desde un 
	 * filtro, ser ejecutado siempre que llegue una peticin de un cliente y antes de que la 
	 * peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al 
	 * cliente. Comprueba si se ha pulsado el botn de eliminar tems, el de aadir tems  o el 
	 * de introducir los pesos a los tems ya aadidos.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			si hay algn problema con el acceso a los ficheros de los 
	 * 								tems
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see 	control.ControlFilter
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EnteroPositivo
	 * @see		xml.items.ChoiceXML
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		HttpSession sesion = request.getSession();
		ServletContext sc =	sesion.getServletContext();
		// Directorio base de trabajo (guardado en "web.xml")
		String dirTrabajo = sc.getInitParameter("Directorio Trabajo");
		// Directorio de asignatura si venimos de la pgina de seleccin de asignatura
		String dirAsignatura = request.getParameter("DirectorioAsignatura");
		// Directorio seleccionado a mostrar en este momento
		String dirItemActual = request.getParameter("DirectorioActual");
		// Directorio inicial de los tems (guardado en "web.xml")
		String dirInicial = sc.getInitParameter("Directorio Items");
		// Directorio superior del actual (null si estamos en el inicial)
		String dirSuperior = null;
		// Si vamos a otra pgina o seguimos en la misma
		boolean salida = false;
		// Estado de la web y mensajes para el usuario en caso de error
		MensajeEstado msg = new MensajeEstado();

		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){
			if (dirItemActual == null){
				// Primera vez que se entra en la pgina, establece el directorio de inicio
				dirItemActual = "/" + dirAsignatura + dirInicial;
				// Comprueba si existe el directorio base de los tems, y si no existe intenta 
				// crearlo, para mostrarlo aunque sea vaco de tems
				File dirItems = new File(sc.getRealPath(dirTrabajo) + "/" + dirAsignatura + 
						dirInicial);
				if (!dirItems.exists())
					if (!dirItems.mkdir())
						throw new IOException("Error: No se pudo crear el directorio base de " +
							"los tems");
			}

			// Recupera de sesin el vector con los tems que ya estn a aadir
			Vector /* ResumenItem */ itemsAAadir = 
					(Vector)sesion.getAttribute("Items_A_Aadir");
			// Si an no se ha creado el vector de los tems a aadir se crea uno vaco y se 
			// guarda en sesin para utilizarlo
			if (itemsAAadir == null){
				itemsAAadir = new Vector();
				sesion.setAttribute("Items_A_Aadir", itemsAAadir);
			}
		
			// Si se ha pulsado el botn "Aadir" para agregar algn tem al test
			if (request.getParameter("Aadir") != null){
				// Se llama al mtodo para aadir los tems elegidos
				aadeItem(request);
			}
		
			// Si se ha pulsado el botn "Eliminar" para borrar algn tem del test
			if (request.getParameter("Eliminar") != null){
				// Se llama al mtodo para eliminar los tems elegidos
				eliminaItem(request);
			}
		
			// Si se ha pulsado el botn "Introducir Pesos"
			if (request.getParameter("Pesos") != null){
				// Comprueba que el nmero de tems a aadir al test sea mayor que cero
				if (itemsAAadir.size() > 0){
					// Comprueba que el nmero de tems que se seleccionan, si se realiza alguna
					// seleccin, sea igual o mayor que el nmero de tems a aadir
					AssessmentTest test = (AssessmentTest) sesion.getAttribute("Test1");
					Selection sel = test.getTestPart().getSeccion().getSeleccion();
					if (sel != null){
						int select = sel.getSeleccion();
						if (select > itemsAAadir.size()){
							msg.setEstado(false);
							msg.setMensaje("Debe seleccionar un nmero de tems mayor o igual" +
									" que el nmero de tems a seleccionar, " + select + 
									"<br />");
						}
					}
				} else {
					msg.setEstado(false);
					msg.setMensaje("El nmero de tems del test no puede ser cero<br />");
				}
				if (msg.isEstado() == true) {
					// Se borra de sesin el vector de los tems disponibles para aadir que ya 
					// no es necesario
					sesion.removeAttribute("Items");
					// Convierte el Vector con los tems a aadir en un array pues su tamao ya 
					// no se va a modificar ms y lo almacena en sesin para utilizarlo luego
					ResumenItem[] riArray = new ResumenItem[itemsAAadir.size()];
					riArray = (ResumenItem[])itemsAAadir.toArray(riArray);
					sesion.removeAttribute("Items_A_Aadir");
					sesion.setAttribute("Items_A_Aadir", riArray);
					// Se redirecciona la respuesta a la pgina siguiente
					response.sendRedirect("introducirpesos.jsp");
					salida = true;
				}
			}
		} else	{		// Si la sesin ha expirado
			response.sendRedirect("errorfinsesion.jsp");
			salida = true;
		}

		// Si se sigue en esta pgina muestra los directorios para poder navegar por ellos
		if (salida != true){
			// Busca la segunda localizacin del carcter '/' para ver si estamos en el inicio
			int loc = dirItemActual.indexOf('/', 1);
			// Si longitud de la direccin actual es distinta de la que hay del inicio al 
			// segundo  carcter '/' ms la del directorio inicial de los tems no estamos en 
			// el base, y por lo tanto mostramos la opcin de ir al directorio superior
			if (loc + dirInicial.length() != dirItemActual.length()) {
				// Si no estamos en el inicial guarda el nombre del directorio padre para 
				// poder volver atrs
				dirSuperior = dirItemActual.substring(0, dirItemActual.lastIndexOf('/'));
			}
			
			// Guarda en request los nombres del directorio actual y el del directorio padre
			request.setAttribute("DirectorioActual", dirItemActual);
			request.setAttribute("DirectorioSuperior", dirSuperior);
			
			// Lista los nombres de los directorios del directorio actual y los guarda en 
			// request
			File dirActual = new File(sc.getRealPath(dirTrabajo) + dirItemActual);
			Vector /* String */ dirNames = ListaArchivos.ListaDirectorios(dirActual);
			request.setAttribute("NombresDirectorios",dirNames);
			
			// Lista los nombres y los ttulos de los tems del directorio actual y los guarda
			// en sesin para aadir el correspondiente al seleccionado en la pgina
			Vector /* ResumenItem */ items = ListaArchivos.ListaItems(dirActual);
			sesion.setAttribute("Items", items);
			
			// Boolean con los tems que estn ya seleccionados de la lista de disponibles
			boolean[] itemsSeleccionados = marcaItemsSeleccionados(items, request);
			
			// Guarda en request el array marcando los tems de los disponibles que ya se han 
			// seleccionado
			request.setAttribute("Seleccionados", itemsSeleccionados);
			
			// Guarda en request los mensajes de aviso para el usuario
			request.setAttribute("Aviso", msg.getMensaje());
		}
		
		return true;
	}
	
	/**
	 * Aade los tems que se han seleccionado en la pgina marcando sus checkboxes, a la lista 
	 * de tems a aadir que se encuentra en sesin. Recibe slo el parmetro <code>request
	 * </code> para acceder a la peticin que guarda los tems marcados, y a sesin, donde se 
	 * guarda la lista de tems a aadir.
	 * 
	 * @param 	request		peticin del usuario para acceder a los tems marcados y a la sesin
	 */
	private void aadeItem(HttpServletRequest request){
		// Objeto sesin para acceder a los objetos ah almacenados
		HttpSession sesion = request.getSession();
		// Recupera los tems que estn actualmente disponibles para aadir
		Vector /* ResumenItem*/ itemsDisponibles = (Vector)sesion.getAttribute("Items");
		// Se recupera de sesin el vector con los tems ya elegidos que estn a aadir
		Vector /* ResumenItem */ itemsAAadir = (Vector)sesion.getAttribute("Items_A_Aadir");
		// Nmero de tems aadidos
		int num = 0;
		// Comprueba todos los tems a ver si se ha marcado alguno para aadir
		for (int i = 0; i < itemsDisponibles.size(); i++){
			String check = request.getParameter("Aade_" + i);
			// Si ese tem se ha marcado para aadir
			if (check != null && check.equalsIgnoreCase("ON")){
				// Se recupera el tem a aadir, que est en la misma posicin en el vector 
				// que la posicin indicada por el checkbox marcado
				ResumenItem ri = (ResumenItem)itemsDisponibles.elementAt(i);
				// Se aade el tem al vector de los tems a aadir
				itemsAAadir.add(ri);
				num++;
			}
		}
		// Mensajes de informacin al usuario
		if (num > 1)
			request.setAttribute("Informacion", "tems aadidos con xito<br />");
		else if(num == 1)
			request.setAttribute("Informacion", "tem aadido con xito<br />");
		else
			request.setAttribute("Informacion", "No ha seleccionado ningn tem<br />");
		
		// Deja en sesin el Vector de tems a aadir para terminar de aadirlos al Test en la 
		// pgina siguiente
		sesion.setAttribute("Items_A_Aadir", itemsAAadir);
	}
	
	/**
	 * Elimina los tems que se han seleccionado en la pgina marcando sus checkboxes, de la 
	 * lista de tems a aadir que se encuentra en sesin. Recibe slo el parmetro <code>
	 * request</code> para acceder a la sesin, donde se guarda la lista de tems a aadir.
	 * 
	 * @param 	request		peticin del usuario para acceder a los tems marcados y a la sesin
	 */
	private void eliminaItem(HttpServletRequest request){
		// Objeto sesin para acceder a los objetos ah almacenados
		HttpSession sesion = request.getSession();
		// Recupera los tems que estn elegidos actualmente a aadir
		Vector /* ResumenItem */ itemsElegidos = (Vector)sesion.getAttribute("Items_A_Aadir");
		// Nmero de tems eliminados
		int num = 0;
		// Nmero de tems a comprobar
		int numItems = itemsElegidos.size();
		// Comprueba todos los tems a ver si se ha marcado alguno para eliminar
		for (int i = 0; i < numItems; i++){
			String check = request.getParameter("Elimina_" + i);
			// Si ese tem se ha marcado para eliminar
			if (check != null && check.equalsIgnoreCase("ON")){
				// Se elimina el tem, que est en la misma posicin en el vector 
				// que la posicin indicada por el checkbox marcado
				itemsElegidos.remove(i - num);
				num++;
			}
		}
		// Mensajes de informacin al usuario
		if (num > 1)
			request.setAttribute("Informacion", "tems eliminados con xito<br />");
		else if(num == 1)
			request.setAttribute("Informacion", "tem eliminado con xito<br />");
		else
			request.setAttribute("Informacion", "No ha seleccionado ningn tem<br />");

		// Deja en sesin el Vector de tems a aadir para terminar de aadirlos al Test en la 
		// pgina siguiente
		sesion.setAttribute("Items_A_Aadir", itemsElegidos);
	}
	
	/**
	 * Rellena un array de <code>boolean</code> indicando con true la posicin de los elementos 
	 * del vector de tems disponibles para seleccionar que ya se han seleccionado. Mediante la 
	 * peticin se accede al objeto de sesin para recoger el objeto de tems seleccionados a 
	 * aadir.
	 *  
	 * @param 	itemsDisponibles	vector de tems disponibles para aadir
	 * @param 	request				peticin del usuario para acceder a los tems seleccionados 
	 * 								a aadir
	 * @return						array de boolean indicando qu tems se han seleccionado ya
	 */
	private boolean[] marcaItemsSeleccionados(Vector itemsDisponibles, 
			HttpServletRequest request) {

		// Array de boolean para marcar los elementos del vector de tems disponibles que ya se 
		// han seleccionado
		boolean[] seleccionados = new boolean[itemsDisponibles.size()];
		// Objeto sesin para acceder a los objetos ah almacenados
		HttpSession sesion = request.getSession();
		// Recupera los tems que estn elegidos actualmente a aadir
		Vector /* ResumenItem */ itemsElegidos = (Vector)sesion.getAttribute("Items_A_Aadir");
		
		// Comprueba qu tems de los disponibles estn ya seleccionados
		for (int i = 0; i < itemsDisponibles.size(); i++){
			ResumenItem itemDisp = (ResumenItem)itemsDisponibles.elementAt(i);
			// Nombre del tem disponible
			String disponible = itemDisp.getArchivo().getPath();
			boolean encontrado = false;
			for (int j = 0; j < itemsElegidos.size() && encontrado == false; j++){
				ResumenItem itemEle = (ResumenItem)itemsElegidos.elementAt(j);
				// Nombre del tem elegido a aadir
				String elegido = itemEle.getArchivo().getPath();
				if (disponible.equals(elegido)){
					seleccionados[i] = true;
					encontrado = true;
				}
				else
					seleccionados[i] = false;
			}
		}
		return seleccionados;
	}
}