package control;

import utilidades.MensajeEstado;
import utilidades.EstadoWeb;
import utilidades.ParserNeutro;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import xml.items.TextentryXML;

/**
 * Componente de control implcito asociado a la pgina JSP "textentry.jsp". Es la lgica 
 * asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea 
 * llamada la pgina. Su funcin es comprobar que todos los parmetros iniciales de la toma de 
 * datos de una pregunta tipo "Text Entry" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la siguiente pgina de toma de datos de tipo "Text Entry". La clase implementa 
 * la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el 
 * nico que contiene, y que es el encargado de realizar la lgica asociada a la pgina 
 * "textentry.jsp".
 * 
 * @author 	David Domnguez
 * @see		Control
 * 
 */
public class Textentry implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina "textentry.jsp". 
	 * Ser llamado por el filtro <code>ControlFilter</code> en el caso de que detecte que la 
	 * pgina requerida es "textentry.jsp". Ya que es llamado desde un filtro, ser ejecutado 
	 * siempre que llegue una peticin de un cliente y antes de que la peticin llegue al 
	 * Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. Comprueba 
	 * uno por uno todos los parmetros, y, en caso de que sean correctos, crea un objeto <code>
	 * TextentryXML</code> para guardar los datos asociados al tipo de pregunta y lo guarda en 
	 * el mbito de sesin para utilizarlos ms adelante para terminar de crear el tem.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		xml.items.TextentryXML
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Comprueba que no sea la primera vez que se visita la pgina
		if (request.getParameter("Form") != null){

			// Caracteres reservados no permitidos
			final String reservados = ";/?:@&=+$,\\*\"<>|";

			String identificador = request.getParameter("Identif");	// Identificador del tem
			String titulo = request.getParameter("Tit");			// Ttulo del tem
			String instrucciones = request.getParameter("Instruc");	// Instrucciones del tem

			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			// Comprueba que estn introducidos todos los atributos obligatorios
			if (identificador != null && !identificador.equals("") &&
					titulo != null && !titulo.equals("")){

				// Comprueba que los caracteres del identificador estn permitidos
		        for(int i = 0; i < identificador.length() && estado.isEstado() == true; i++) {
		        	char c = identificador.charAt(i);	// Lee el siguiente carcter
		        	int marca = reservados.indexOf(c);
		        	if (marca >= 0 ){
						estado.aadeError("Error: El carcter '" + reservados.charAt(marca) + 
								"' en la posicin " + (i+1) + " del Identificador no est " +
								"permitido<br />", "Identif");
		        	}
		        }

				// Comprueba que las instrucciones sean un cdigo XHTML vlido
				if (instrucciones != null && !instrucciones.equals("")){
					MensajeEstado msg = ParserNeutro.compruebaInstruccionesXHTML(instrucciones);
					if (msg.isEstado() == false)
						estado.aadeError(msg.getMensaje(), "Instruc");
				}

				// Si el estado es todo correcto pasa a generar el objeto contenedor del tem 
				if ( estado.isEstado()){
					// Crea un objeto que contiene las caractersticas del tem Text Entry a 
					// crear
					TextentryXML teXML = new TextentryXML(identificador, titulo, 
							instrucciones);
					// Guarda el objeto Textentry en sesin para seguir utilizndolo luego
					HttpSession sesion = request.getSession();
					sesion.setAttribute("TextEntry", teXML);
					// Redirecciona a la pgina de introducir la respuesta
					response.sendRedirect("respuestastextentry.jsp");
				}
			}
			// si no estn todos los atributos obligatorios comprueba los que fallan
			// para informar al usuario
			else{
				if (identificador.equals(""))
					estado.aadeError("Debe introducir un Identificador<br />", "Identif");
				if (titulo.equals(""))
					estado.aadeError("Debe introducir un Ttulo<br />", "Tit");
			}
			// Si ha habido algn fallo pone los datos que ya ha introducido el usuario junto 
			// a los mensajes de aviso y el foco en el primer fallo encontrado
			if (estado.isEstado() == false){
				request.setAttribute("Aviso",estado.getMensaje());
				request.setAttribute("Focus",estado.getFocusON());
				request.setAttribute("Identif",identificador);
				request.setAttribute("Tit",titulo);
				request.setAttribute("Instruc",instrucciones);
			}
		} else {
			// Si es la primera vez que se visita la pgina borra el objeto textentryXML del 
			// mbito de sesin para iniciar desde cero el tem y pone el foco en el primer 
			// campo del formulario
			HttpSession sesion = request.getSession();
			sesion.removeAttribute("TextEntry");
			request.setAttribute("Focus", "Identif");
		}
		
		return true;
	}
}