package utilidades;

import javax.servlet.http.HttpServletRequest;

/**
 * Clase que almacena la representacin de un tipo <i>identifier</i> vlido segn la 
 * norma QTI del IMS. Contiene mtodos para comprobar si es vlido, para obtener el 
 * identificador, un mtodo esttico para comprobar que en un grupo de identificadores sean 
 * todos distintos entre s, y otro mtodo esttico para leer un grupo de identificadores del 
 * request y comprobar que sean vlidos.
 * 
 * @author David Domnguez
 *
 */
public class Identificador {
	
	/**
	 * Variable de clase con el identificador String genrico
	 */
	private String identificador;

	/**
	 * Constructor de la clase en el que simplemente se inicializa el valor de la variable de 
	 * clase con el valor recibido en el parmetro.
	 * 
	 * @param 	identificador	string con el valor inicial para la variable de la clase
	 */
	public Identificador(String identificador) {
		super();
		this.identificador = identificador;
	}

	/**
	 * Mtodo "get" para obtener el valor de la variable de clase. Si no se usa el mtodo <code>
	 * esIdentificador</code> antes no se tendr certeza de si la variable es un identificador 
	 * vlido o no.
	 * 
	 * @return					valor del string del identificador
	 */
	public String getIdentificador() {
		return identificador;
	}
	
	/**
	 * Comprueba si el identificador almacenado en la variable <code>identificador</code> de 
	 * tipo <code>String</code> de la clase es un identificador vlido segn la especificacin 
	 * del QTI. Comprueba que tenga menos de 32 caracteres, que el primer carcter sea una 
	 * letra o '_' (guin bajo), y que el resto de los caracteres sean letras, dgitos,'-' 
	 * (guin), o '_' (guin bajo). En caso de error devuelve un mensaje con la posicin del 
	 * primer carcter errneo encontrado.
	 * 
	 * @return					mensaje con el resultado de la comprobacin y los posibles 
	 * 							avisos al usuario en caso de que haya error
	 * @see 	utilidades.MensajeEstado
	 */
	public MensajeEstado esIdentificador(){
		
		MensajeEstado msg = new MensajeEstado();		// resultado de la comprobacin

		// Comprueba que el string a comprobar tenga algn valor
		if (identificador != null && !identificador.equals("")){
			// Comprueba que la longitud sea menor de 32 caracteres por compatibilidad con la 
			// versin 1 del QTI
			if (identificador.length() <= 32){
				// Comprueba que el primer caracter del identificador sea una letra o '_'
				if (identificador.charAt(0) == '_' || 
						Character.isLetter(identificador.charAt(0))){
					for (int i = 1; i < identificador.length()&& msg.isEstado(); i++){
						// Si algn carcter no es del juego de caracteres permitidos
						if (identificador.charAt(i) != '_' && identificador.charAt(i) != '-' &&
								!Character.isLetterOrDigit(identificador.charAt(i))) {
							int loc = i + 1;
							msg.setEstado(false);
							msg.setMensaje("el carcter en la posicin " + loc + " no est " +
									"permitido");
						}
					}
				} else {
					msg.setEstado(false);
					msg.setMensaje("el primer carcter debe ser una letra o '_'");
				}
			} else {
				msg.setEstado(false);
				msg.setMensaje("debe reducir el nmero de caracteres a 32 o menos");
			}
		} else {
			msg.setEstado(false);
			msg.setMensaje("no puede quedar vaco");
		}
		
		return msg;
	}

	/**
	 * Mtodo esttico para comprobar que los identificadores de una tabla de objetos <code>
	 * Identificador</code> sean todos diferentes entre s.
	 * 
	 * @param 	tabla		tabla de objetos identificador a comprobar
	 * @return				un mensajeEstado con el estado de la comprobacin y los mensajes 
	 * 						pertinentes
	 */
	public static MensajeEstado compruebaIdentificadores(Identificador[] tabla){
		MensajeEstado msg = new MensajeEstado();
		for (int i = 0; i < tabla.length; i++)
			for (int j = i + 1; j < tabla.length; j++)
				if (tabla[i].getIdentificador().equals(tabla[j].getIdentificador())){
					// Si dos identificadores son iguales
					msg.setMensaje("el identificador " + (i+1) + " es igual al identificador "
							+ (j+1));
					msg.setEstado(false);
				}
		
		return msg;
	}
	
	/**
	 * Mtodo esttico para leer un grupo de identificadores recibidos como parmetros en el 
	 * request. Lee todos los identificadores, inicializando  el array de <code>Identificador
	 * </code>, comprueba que sean correctos y modifica si es necesario el objeto <code>
	 * EstadoWeb</code> del estado de la pgina avisando en caso de error.
	 * 
	 * @param 	identificadores		array de objetos identificador a rellenar con los 
	 * 								identificadores	del request
	 * @param 	origen				string con el nombre base de los campos donde se leen en 
	 * 								la web los identificadores 
	 * @param 	estado				estadoWeb, representando el estado de la web. Se modifica 
	 * 								con el estado tras la comprobacin de los identificadores.
	 * @param 	request				peticin recibida del cliente donde estn almacenados los 
	 * 								identificadores introducidos por el usuario
	 * @param	pre					Texto previo para los mensajes de error
	 * @see		javax.servlet.http.HttpServletRequest#getParameter
	 * @see		utilidades.EstadoWeb#aadeError(String, String)
	 * @see		utilidades.MensajeEstado
	 */
	public static void testIdentificadores(Identificador[] identificadores, String origen, 
			EstadoWeb estado, HttpServletRequest request, String pre){
		
		// Lee los identificadores de request con el nombre base comn
		MensajeEstado msg = new MensajeEstado();
		int i;
		for (i = 0; i < identificadores.length; i++)
			identificadores[i]= new Identificador(request.getParameter(origen + i));
		// Comprueba que sean identificadores vlidos
		for (i = 0; i < identificadores.length && msg.isEstado(); i++)
			msg = identificadores[i].esIdentificador();
		if (msg.isEstado()){
			// Si todos son identificadores correctos comprueba que sean todos distintos
			msg = Identificador.compruebaIdentificadores(identificadores);
			if (!msg.isEstado())	// Si hay dos identificadores repetidos
				estado.aadeError("Error: " + pre + msg.getMensaje() + "<br />", origen + "0");
		} else						// Si encuentra un identificador no vlido
			estado.aadeError("Error: " + pre + "el identificador " + i + " " + 
					msg.getMensaje() +	"<br />", origen + (i-1));
	}
}