package utilidades.etiquetas.util.logging;

import javax.servlet.*;

import java.util.logging.*;

/**
 * Arranca y detiene el <code>Logger</code> de la Aplicacin Web cuando sta se arranca y se 
 * detiene. La clase implementa la interfaz <code>ServletContextListener</code> y ejecutar 
 * sus mtodos siempre que el contexto de la Aplicacin Web se cree o se destruya (normalmente 
 * al arrancar o parar la Aplicacin). Se encarga de preparar el objeto que se ocupa de las 
 * acciones relacionadas con el registro de los errores que se produzcan en la Aplicacin, es 
 * decir, el "logging".
 * Contiene una variable de clase <code>String</code> con el nombre del logger de la Aplicacin 
 * para acceder a l desde otros puntos de la Aplicacin por su nombre, y otra de tipo <code>
 * Level</code> donde se define el nivel mnimo de importancia de registro de los errores de la 
 * Aplicacin.
 * 
 * @author 	David Domnguez
 * @see		ServletContextListener
 *
 */
public class AplicacionLogger implements ServletContextListener {

	/**
	 * Nombre del logger de la Aplicacin para acceder a l por su nombre en cualquier punto de 
	 * la Aplicacin que se requiera 
	 */
	static String nombreLogger = "Aplicacion.QTI";
	/**
	 * Nivel mnimo de importancia de los errores que sern guardados por el logger
	 */
	static Level nivelLogger = Level.WARNING;
	
	/**
	 * Mtodo que arranca el <code>Logger</code> de la Aplicacin Web. El mtodo se ejecutar 
	 * siempre que se cree el contexto de la Aplicacin (normalmente al arrancar sta). Se 
	 * encarga de crear un objeto <code>Logger</code> para crear un registro de los errores que 
	 * ocurran en la Aplicacin, estableciendo su nivel mnimo de registro. El manejador del 
	 * Logger se deber aadir siempre que se vaya a realizar algn registro con el Logger.
	 * 
	 * @param 	sce		sevletContextEvent con los datos del contexto que se ha inicializado
	 * @see javax.servlet.ServletContextListener#contextInitialized(javax.servlet.ServletContextEvent)
	 * @see	java.util.logging.Logger
	 */
	public void contextInitialized(ServletContextEvent sce) {

		// Crea una instancia de un Logger
		Logger logger = Logger.getLogger(nombreLogger);
		// Establece el nivel mnimo para almacenar los informes
		logger.setLevel(nivelLogger);
		
		// El manejador del loggger se aade en la ejecucin de la etiqueta en caso de que 
		// ocurra algn error y se vaya a registrar
	}

	/**
	 * Mtodo que detiene el <code>Logger</code> de la Aplicacin Web. El mtodo se ejecutar 
	 * siempre que se destruya el contexto de la Aplicacin (normalmente al detenerla). Se 
	 * encarga de cerrar todos los <code>Handler</code>s y eliminarlos del <code>Logger</code>.
	 * 
	 * @param 	sce		sevletContextEvent con los datos del contexto que se ha inicializado
	 * @see javax.servlet.ServletContextListener#contextDestroyed(javax.servlet.ServletContextEvent)
	 * @see	java.util.Handler
	 */
	public void contextDestroyed(ServletContextEvent sce) {
		// Se obtiene el Logger ya creado por el nombre
		Logger logger = Logger.getLogger(nombreLogger);
		// Array con los Handlers aadidos al logger
		Handler[] handlers = logger.getHandlers();
		// Cada handler se vaca, se cierra y se elimina del logger
		for (int i = 0; i < handlers.length; i++){
			handlers[i].flush();
			handlers[i].close();
			logger.removeHandler(handlers[i]);
		}
	}
}