package utilidades.etiquetas.util.logging;

import java.io.StringWriter;
import java.io.PrintWriter;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;
import java.util.Date;

/**
 * <code>Formater</code> personalizado para realizar los registros de la Aplicacin Web con el 
 * formato personalizado. La clase extiende a la clase abstracta <code>Formatter</code>, 
 * sobreescribiendo slo su mtodo <code>format</code>, aplicando un formato personalizado al 
 * registro de los errores de la Aplicacin.
 * 
 * @author 	David Domnguez
 * @see		java.util.logging.Formatter
 */
public class FormatterPersonalizado extends Formatter {

	/**
	 * Aplica un formato personalizado a la informacin sobre el error a registrar recibida en 
	 * el <code>LogRecord</code>. Pone inicialmente el nivel de importancia del registro con la 
	 * fecha y la hora entre corchetes aplicndole un formato especfico (con los nombres 
	 * cortos de los das de la semana y los meses en castellano); luego el mensaje del 
	 * registro recibido; en la lnea siguiente la uri que ha disparado el registro en el 
	 * servidor; y finalmente la traza del error si ha sido una excepcin.
	 * 
	 * @param	logRecord	logRecord con la informacin a formatear para registrar
	 * @return				string con el formato final de la informacin a registrar
	 * @see java.util.logging.Formatter#format(java.util.logging.LogRecord)
	 */
	public String format(LogRecord logRecord) {
		// Objeto con los smbolos a utilizar para crear una fecha
		DateFormatSymbols dfs = new DateFormatSymbols();
		// Se modifica el objeto de smbolos de fecha poniendo los nombres cortos a utilizar 
		// en castellano
		LoggerTag.setNombresCortosCastellano(dfs);
		// Objeto para formatear una fecha segn el formato indicado y con los smbolos que 
		// indica el objeto de smbolos de fecha
		SimpleDateFormat sdf = new SimpleDateFormat("EEE d MMM HH:mm:ss zzz yyyy", dfs);
		// Objeto con la fecha indicada por el logRecord
		Date date = new Date(logRecord.getMillis());
		// Nombre del nivel de importancia del registro a escribir
		String level = logRecord.getLevel().getName();
		// Primera lnea de la cadena con la informacin formateada con el nivel y la fecha
		// segn el formato indicado
		String cadena = "[" + level + " " + sdf.format(date) + "]\n";
		// Se aade la segunda lnea con el mensaje informando del registro recibido
		cadena += logRecord.getMessage() + "\n";
		// Se obtiene la direccin de la uri que dispar el registro del parmetro en la 
		// posicin #0 del array de parmetros
		String origen = logRecord.getParameters()[0].toString();
		if (origen != null)
			cadena += "URI que provoc el error: " + origen;
		// Se aade si el registro implica una excepcin la traza de sta
		Throwable thrown = logRecord.getThrown();
		if (thrown != null) {
			StringWriter sw = new StringWriter();
			thrown.printStackTrace(new PrintWriter(sw));
			cadena += "\n" + sw.toString();
		}
		cadena +=  "\n\n";
		
		return cadena;
	}
}