package utilidades.etiquetas.util.logging;

import java.io.File;
import java.io.IOException;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.logging.*;

import javax.servlet.jsp.tagext.SimpleTagSupport;
import javax.servlet.jsp.*;

/**
 * Etiqueta de pgina JSP para realizar el registro de algn error o evento que se requiera 
 * registrar de la Aplicacin Web. La clase extiende a la clase <code>SimpleTagSupport</code> 
 * que representa a una Etiqueta Personalizada de JSP Simple. La etiqueta se encarga de 
 * registrar algn error o informacin en el <code>Logger</code> que se cre al iniciar la 
 * Aplicacin Web. Tiene dos variables de clase que son los dos atributos que se le pasan a la 
 * etiqueta, con sus dos mtodos "setters" correspondientes: el primero de ellos es un <code>
 * String</code> con el nivel de importancia del registro a realizar, y el segundo el mensaje 
 * del registro. Todo el proceso de registro se realiza en el mtodo sobreescrito <code>
 * doTag</code>, que se ejecuta al llamar a la etiqueta en una pgina JSP.
 * 
 * Contiene tambin un mtodo esttico para poner en castellano los nombres cortos de los das 
 * de la semana y de los meses del ao de un objeto de formato de los smbolos de fechas, 
 * <code>DateFormatSymbols</code>. 
 * 
 * @author 	David Domnguez
 * @see		javax.servlet.jsp.tagext.SimpleTagSupport
 */
public class LoggerTag extends SimpleTagSupport {

	/**
	 * String con el nombre del nivel de importancia del registro.
	 */
	private String nivel;
	/**
	 * String con el mensaje a registrar.
	 */
	private String mensaje;

	
	/**
	 * Mtodo "set" que establece el mensaje a registrar con el <code>Logger</code>.
	 * 
	 * @param 	mensaje		string con el mensaje a registrar
	 */
	public void setMensaje(String mensaje) {
		this.mensaje = mensaje;
	}


	/**
	 * Mtodo "set" que establece el nombre del nivel de importancia del registro a realizar.
	 * 
	 * @param 	nivel		string con el nombre del nivel de importancia del registro
	 */
	public void setNivel(String nivel) {
		this.nivel = nivel;
	}


	/**
	 * Realiza el registro de la informacin requerida en el <code>Logger</code> de la 
	 * Aplicacin. El mtodo se ejecuta siempre que se llama a la etiqueta personalizada 
	 * asociada a esta clase. Se encarga de asociar y configurar un manejador con el <code>
	 * Logger</code> de la Aplicacin para registrar la informacin recibida en la pgina desde 
	 * la que se llama a esta etiqueta, y la indicada en los atributos, mediante ese manejador, 
	 * o <code>Handler</code>.
	 *  
	 * @see 	javax.servlet.jsp.tagext.SimpleTag#doTag()
	 * @see 	java.util.logging.Logger
	 * @see 	java.util.logging.Handler
	 * @see		java.text.DateFormatSymbols
	 * @see		java.text.SimpleDateFormat
	 */
	public void doTag(){
		// Directorio en el que almacenar los registros
		final String dirRegistro = "/WEB-INF/log/";
		// Manejador de mensajes; los recibe del logger y los escribe en un archivo
		FileHandler fh = null;
		// Contexto de la pgina para acceder a la direccin real del directorio de registro
		PageContext pc = (PageContext)getJspContext();
		// Nombre del directorio donde almacenar los registros
		String root = pc.getServletContext().getRealPath(dirRegistro);
		File dir = new File(root);
		// Si el directorio no existe intenta crearlo
		boolean dirExiste = dir.exists(); 
		if (!dirExiste)
			dirExiste = dir.mkdirs();
		
		//Si el directorio existe
		if (dirExiste){
			// Objeto con los smbolos a utilizar para crear una fecha
			DateFormatSymbols dfs = new DateFormatSymbols();
			// Se modifica el objeto de smbolos de fecha poniendo los nombres cortos a 
			// utilizar en castellano
			setNombresCortosCastellano(dfs);
			// Objeto para formatear una fecha segn el formato indicado y con los smbolos que 
			// indica el objeto de smbolos de fecha
			SimpleDateFormat sdf = new SimpleDateFormat("EEE, d-MMM-yyyy", dfs);
			// Objeto con la fecha actual
			Date date = new Date();
			// Nombre del archivo de registro con la fecha actual
			String nombre = sdf.format(date) + ".log";
			try {
				// Se recupera la instancia del logger creada por el nombre
				Logger logger = Logger.getLogger(AplicacionLogger.nombreLogger);
				// File del archivo de registro
				File logTxt = new File(root + "\\" + nombre);
				// Se recupera el array de Handlers asociados al logger
				Handler[] handlers = logger.getHandlers();
				// Si el archivo no existe o si el logger no tiene handlers asociados se crea
				// el manejador apropiado para el logger
				if (!logTxt.exists() || handlers.length == 0 ){
					// Nuevo manejador de registro en fichero en el fichero que corresponde, e 
					// indicando que los nuevos registros se aaden al final
					fh = new FileHandler(root + "\\" + nombre, true);
					// Se establece el administrador de errores del handler que capturar sus 
					// excepciones y acta en consecuencia
					fh.setErrorManager(new ErrorManagerPersonalizado());
					// Establece el objeto Formmatter que aplica el formato a la informacin a 
					// registrar en el archivo
					fh.setFormatter(new FormatterPersonalizado());
					// Vaca y cierra los manejadores que estn asociados al logger y los 
					// disocia de l
					for (int i = 0; i < handlers.length; i++){
						handlers[i].flush();
						handlers[i].close();
						logger.removeHandler(handlers[i]);
					}
					// Aade el nuevo handler de archivo creado al logger
					logger.addHandler(fh);
				}
				// Se crea un objeto registro con la informacin a almacenar de los atributos
				LogRecord record = new LogRecord(Level.parse(nivel), mensaje);
				// Se almacena en el objeto registro el objeto Throwable contenido en la pgina 
				// que llama a la etiqueta si lo hay, almacenado en "request" en caso de error
				Throwable thrown = (Throwable)pc.getAttribute("javax.servlet.error.exception", 
						PageContext.REQUEST_SCOPE);
				if (thrown != null)
					record.setThrown(thrown);
				// Ms parmetros a aadir al objeto registro en un array
				Object[] params = new Object[1];
				// Se aade slo la uri que ha causado el error que est almacenada en "request"
				params[0] = (String)pc.getAttribute("javax.servlet.error.request_uri", 
						PageContext.REQUEST_SCOPE);
				// Se aade el array de parmetros al objeto registro
				record.setParameters(params);
				// Se enva la informacin a registrar al logger
				logger.log(record);
			} catch (IOException e){
				// Si no se ha podido cargar el logger se avisa por la salida estndar de 
				// errores
				System.err.println("No se pudo cargar el logger: " +e.getMessage());
			}
		}
		else {
			// si no se ha podido crear el directorio de los registros se avisa por la salida 
			// estndar de errores
			System.err.println("No se ha podido crear el directorio de registro: " + 
					dirRegistro);
		}
	}
	
	
	/**
	 * Personaliza un objeto <code>DateFormatSymbols</code> poniendo algunos nombres cortos en 
	 * castellano. El mtodo esttico se encarga de modificar los nombres cortos de los das de 
	 * la semana y los de los meses del ao de un objeto de formato de los smbolos de las 
	 * fechas, <code>DateFormatSymbols</code>, ponindolos en castellano.
	 * 
	 * @param 	dfs		dateFormatSymbols a modificar
	 */
	public static void setNombresCortosCastellano(DateFormatSymbols dfs){
		// Array con los nombres cortos de los meses del ao indexados segn las constantes de 
		// la clase Calendar
		String[] mesesCortos = new String[12];
		mesesCortos[Calendar.JANUARY] = "Ene";
		mesesCortos[Calendar.FEBRUARY] = "Feb";
		mesesCortos[Calendar.MARCH] = "Mar";
		mesesCortos[Calendar.APRIL] = "Abr";
		mesesCortos[Calendar.MAY] = "May";
		mesesCortos[Calendar.JUNE] = "Jun";
		mesesCortos[Calendar.JULY] = "Jul";
		mesesCortos[Calendar.AUGUST] = "Ago";
		mesesCortos[Calendar.SEPTEMBER] = "Sep";
		mesesCortos[Calendar.OCTOBER] = "Oct";
		mesesCortos[Calendar.NOVEMBER] = "Nov";
		mesesCortos[Calendar.DECEMBER] = "dic";
		
		// Array con los nombres cortos de los das de la semana indexados segn las constantes 
		// de la clase Calendar (de 8 elementos pues las constantes van del 1 al 7, ninguna 
		// vale 0)
		String[] diasCortos = new String[8];
		diasCortos[Calendar.MONDAY] = "Lun";
		diasCortos[Calendar.TUESDAY] = "Mar";
		diasCortos[Calendar.WEDNESDAY] = "Mier";
		diasCortos[Calendar.THURSDAY] = "Jue";
		diasCortos[Calendar.FRIDAY] = "Vie";
		diasCortos[Calendar.SATURDAY] = "Sab";
		diasCortos[Calendar.SUNDAY] = "Dom";
		
		// Establece los nuevos das cortos de los das de la semana y de los meses del ao
		dfs.setShortWeekdays(diasCortos);
		dfs.setShortMonths(mesesCortos);
	}
}