package xml.items;

import utilidades.MensajeEstado;

import java.io.IOException;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import javax.servlet.ServletContext;

/**
 * Representacin de un tem XML de la norma QTI del IMS general. Es una clase abstracta, 
 * superclase de todas las clases contenedoras de los tems XML. Esta clase representa un tem 
 * XML general, y como tal, contiene los tres campos que tienen todos los tems: <code>
 * identificador</code>, <code>ttulo</code>, e <code>instrucciones</code> del tem. Contiene 
 * un constructor que inicializa esos tres campos. Tambin un mtodo abstracto para escribir el 
 * tem XML en disco y un mtodo para escribir la cabecera comn de todos los tems y otro para 
 * el pie.
 *  
 * @author David Domnguez
 *
 */
public abstract class AssessmentItem {

	
	/**
	 * El identificador del tem, que ser tambin el nombre del archivo.
	 */
	protected String identificador;
	/**
	 * Ttulo del tem.
	 */
	protected String titulo;
	/**
	 * Instrucciones para responder el tem.
	 */
	protected String instrucciones;
	
	/**
	 * Constructor que le da valor a las tres variables de clase. Simplemente las inicializa a 
	 * esos valores.
	 * 
	 * @param 	identificador		identificador del tem (y el nombre del archivo)
	 * @param 	titulo				ttulo del tem
	 * @param 	instrucciones		instrucciones para el tem
	 */
	public AssessmentItem (String identificador, String titulo, String instrucciones) {
		
		this.identificador = identificador;
		this.titulo = titulo;
		this.instrucciones = instrucciones;
	}

	/**
	 * Mtodo "get" para obtener el identificador actual de tem. Para comprobar si el nombre 
	 * del fichero ya existe antes de intentar crearlo.
	 * 
	 * @return					string con el identificador actual del tem
	 * @see		#setIdentificador
	 */
	public String getIdentificador() {
		return identificador;
	}

	/**
	 * Mtodo "set" para establecer el identificador del tem. Si el que tiene no es un nombre 
	 * de archivo vlido con este mtodo se le pone un nombre nuevo.
	 * 
	 * @param 	identificador	string con el nuevo identificador
	 * @see		#getIdentificador
	 */
	public void setIdentificador(String identificador) {
		this.identificador = identificador;
	}

	/**
	 * Escribe en disco el tem XML al que representa esta clase. Debe ser sobreescrito por 
	 * todos los tems distintos.El mtodo se encarga de construir un tem XML de acuerdo con 
	 * la norma QTI con los campos que tiene almacenados y en la direccin que recibe como 
	 * parmetro. Devuelve un <code>MensajeEstado</code> informando del resultado del proceso.
	 * 
	 * @param 	path		string con la direccin absoluta en la que crear el archivo XML	
	 * @param	sc			servletContext para acceder a los parmetros iniciales de "web.xml"
	 * @param	req			string con la url base de la Aplicacin Web requerida
	 * @return				un mensajeEstado informando del estado final del proceso
	 * @throws 	IOException	si hay algn problema al intentar escribir el archivo en disco
	 */
	public abstract MensajeEstado creaItemXML(String path, ServletContext sc, String req) 
			throws IOException;
	
	/**
	 * Escribe en un <code>writer</code> especfico para escribir archivos XML la cabecera 
	 * comn a todos los tipos de tem XML.
	 * 
	 * @param 	xsw					XMLStreamWriter, un writer especfico para escribir 
	 * 								contenido XML
	 * @param	sc					servletContext para leer los parmetros del nombre de la 
	 * 								herramienta y la versin de "web.xml"
	 * @throws 	XMLStreamException	si hay algn problema con los datos XML a escribir
	 * @see		XMLStreamWriter
	 */
	protected void writeCabecera(XMLStreamWriter xsw, ServletContext sc) 
			throws XMLStreamException{
		xsw.writeStartDocument("ISO-8859-1","1.0");
		// Elemento raz de todos los tems XML
		xsw.writeStartElement("assessmentItem");
		xsw.writeDefaultNamespace("http://www.imsglobal.org/xsd/imsqti_v2p1");
		xsw.writeNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
		xsw.setPrefix("xsi", "http://www.w3.org/2001/XMLSchema-instance");
		xsw.writeAttribute("http://www.w3.org/2001/XMLSchema-instance", "schemaLocation", 
				"http://www.imsglobal.org/xsd/imsqti_v2p1 imsqti_v2p1.xsd");
		// La implementacin no admite tems adaptativos
		xsw.writeAttribute("adaptive", "false");
		// La implementacin no admite tems dependientes del tiempo
		xsw.writeAttribute("timeDependent", "false");
		// Nombre y versin de esta herramienta de creacin de tems XML
		xsw.writeAttribute("toolName", sc.getInitParameter("nombre herramienta"));
		xsw.writeAttribute("toolVersion", sc.getInitParameter("versin herramienta"));
	}
	
	/**
	 * Escribe en un <code>writer</code> especfico para escribir archivos XML el pie comn a 
	 * todos los tipos de tem XML. Al finalizar llama a <code>flush</code> del <code>writer
	 * </code> y finalmente lo cierra llamando a <code>close</code>.
	 * 
	 * @param 	xsw					XMLStreamWriter, un writer especfico para escribir 
	 * 								contenido XML
	 * @param	sc					servletContext para leer la uri de la plantilla para el 
	 * 								procesado de la respuesta
	 * @param	req					strng con la URL base del proyecto
	 * @throws 	XMLStreamException	si hay algn problema con los datos XML a escribir
	 * @see		XMLStreamWriter
	 */
	protected void writePie(XMLStreamWriter xsw, ServletContext sc, String req) 
			throws XMLStreamException {
		xsw.writeEndElement();							// Cierra "itemBody"
		xsw.writeStartElement("responseProcessing");	// ResponseProcessing de la respuesta
		String template;
		
		// uri de la plantilla de respuesta a partir de la URL base del proyecto ms su uri 
		// relativa
		template = req + sc.getInitParameter("Uri match_response");
		xsw.writeAttribute("template", template);
		
		xsw.writeEndElement();							// Cierra responseProcessing
		xsw.writeEndElement();							// Cierra "assessmentItem"
		xsw.writeEndDocument();							// Fin del documento
		xsw.flush();									// Vaca el Writer
		xsw.close();									// Cierra el Writer
	}
}