package xml.items;

import java.io.*;

import javax.xml.stream.*;
import javax.servlet.ServletContext;

import utilidades.*;

import org.apache.commons.io.FileUtils;

/**
 * Clase contenedora del tem tipo "Choice". Contiene un campo para cada parmetro del tem 
 * necesario para construir el archivo del tem XML. Con los mtodos que incluye permite crear 
 * un tem tipo "Choice" completo y crear el archivo XML en disco.
 * 
 * @author 	David Domnguez
 * @see		Identificador
 */
public class ChoiceXML extends AssessmentItem{
	
	/**
	 * Pregunta del tem.
	 */
	protected String pregunta;
	/**
	 * Mezclar las respuestas del tem s o no.
	 */
	protected boolean shuffle;
	/**
	 * Texto de cada una de las respuestas.
	 */
	protected String[] respuestas;
	/**
	 * Identificador de cada una de las respuestas.
	 */
	protected Identificador[] identificadores;
	/**
	 * Nmero mximo de respuestas seleccionables.
	 */
	protected int maxChoices;
	/**
	 * Array de boolean de respuesta correcta.
	 */
	protected boolean[] correcto;
	/**
	 * Array de boolean de respuesta fija.
	 */
	protected boolean[] fija;
	/**
	 * Cardinalidad de la variable respuesta.
	 */
	protected String cardinalidad;
	
	/**
	 * Constructor que permite inicializar el objeto con un primer grupo de parmetros. Son 
	 * los que se recogen en la primera pgina de la toma de datos. Para los parmetros 
	 * comunes a todos los tems se llama al constructor de la superclase.
	 * 
	 * @param 	identificador	identificador del tem
	 * @param 	titulo			ttulo del tem
	 * @param 	instrucciones	instrucciones del tem
	 * @param 	pregunta		pregunta del tem
	 * @param 	shuffle			mezclar las respuestas o no
	 */
	public ChoiceXML (String identificador, String titulo, String instrucciones, 
			String pregunta, boolean shuffle) {
		super(identificador, titulo, instrucciones);
		this.pregunta = pregunta;
		this.shuffle = shuffle;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>boolean</code> que indica si la 
	 * respuesta en esa misma posicin es correcta.
	 * 
	 * @param 	correcto		array con las respuestas que son correctas
	 */
	public void setCorrecto(boolean[] correcto) {
		this.correcto = correcto;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>boolean</code> que indica si la 
	 * respuesta en esa misma posicin debe quedar fija en el caso de que se tengan que 
	 * mezclar aleatoriamente.
	 * 
	 * @param 	fija			array con las respuestas en posicin fija
	 */
	public void setFija(boolean[] fija) {
		this.fija = fija;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de objetos <code>Identificador</code> que 
	 * contiene en cada posicin el identificador de la respuesta en esa posicin.
	 * 
	 * @param 	identificadores	array con los identificadores de las respuestas
	 */
	public void setIdentificadores(Identificador[] identificadores) {
		this.identificadores = identificadores;
	}

	/**
	 * Mtodo que inicializa el nmero mximo de respuestas que se pueden elegir. En funcin 
	 * del nmero, inicializa el valor de la variable <code>cardinalidad</code>.
	 *  
	 * @param 	maxChoices		nmero mximo de respuestas que se pueden elegir
	 */
	public void setMaxChoices(int maxChoices) {
		this.maxChoices = maxChoices;
		switch (maxChoices){
			case 1:		cardinalidad = "single"; break;
			default: 	cardinalidad = "multiple"; break;
		}			
	}

	/**
	 * Mtodo que inicializa el <code>array</code> con las respuestas posibles a elegir.
	 * 
	 * @param 	respuestas		array con las respuestas
	 */
	public void setRespuestas(String[] respuestas) {
		this.respuestas = respuestas;
	}

	/**
	 * Mtodo que construye el tem XML tipo "Choice"  en la direccin que recibe como 
	 * parmetro con los campos que tiene almacenados. Devuelve informacin de estado.
	 * 
	 * @param 	path		direccin absoluta en la que crear el archivo XML
	 * @param	sc			servletContext para acceder a los parmetros iniciales de "web.xml"
	 * @param	req			string con la url base de la Aplicacin Web requerida
	 * @return				informacin del estado final del proceso
	 * @throws 	IOException	si hay algn problema al intentar escribir el archivo en disco
	 * @see		XMLOutputFactory#createXMLStreamWriter(java.io.Writer)
	 * @see		XMLStreamWriter
	 * @see		StringWriter
	 */
	public MensajeEstado creaItemXML(String path, ServletContext sc, String req) 
			throws IOException{
		
		MensajeEstado msg = new MensajeEstado();		// Mensaje de salida del proceso
		
		try {
			XMLOutputFactory xof =  XMLOutputFactory.newInstance();
			XMLStreamWriter xsw = null;				// Writer para escribir datos XML
			StringWriter sw = new StringWriter();	// Buffer en el que escribir
			xsw = xof.createXMLStreamWriter(sw);	// El writer XML escribe en el buffer
			// Escribe cabecera comn a todos los tipos de pregunta
			writeCabecera(xsw, sc);
			xsw.writeAttribute("identifier", identificador);
			xsw.writeAttribute("title", titulo);
			xsw.writeStartElement("responseDeclaration");	// Declara la variable de respuesta
			xsw.writeAttribute("identifier", "RESPONSE");
			xsw.writeAttribute("cardinality", cardinalidad);
			xsw.writeAttribute("baseType", "identifier");
			xsw.writeStartElement("correctResponse");	// Valores de las respuestas correctas
			for (int i = 0; i < respuestas.length; i++){
				if (correcto[i]){
					xsw.writeStartElement("value");			// Valor correcto
					xsw.writeCharacters(identificadores[i].getIdentificador());
					xsw.writeEndElement();					// Cierra "value"
				}
			}
			xsw.writeEndElement();							// Cierra "correctResponse"
			xsw.writeEndElement();							// Cierra "responseDeclaration"
			xsw.writeStartElement("outcomeDeclaration");	// Variable de puntuacin
			xsw.writeAttribute("identifier", "SCORE");
			xsw.writeAttribute("cardinality", "single");
			xsw.writeAttribute("baseType", "integer");
			xsw.writeStartElement("defaultValue");			// Valor por defecto de la salida
			xsw.writeStartElement("value");					// Valor
			xsw.writeCharacters("0");
			xsw.writeEndElement();							// Cierra "value"
			xsw.writeEndElement();							// Cierra "defaultValue"
			xsw.writeEndElement();							// Cierra "outcomeDeclaration"
			xsw.writeStartElement("itemBody");				// Cuerpo del tem
			// Las instrucciones se escriben mediante el parser neutro puesto que es XHTML, sin 
			// que el writer XML modifique los datos
			if (instrucciones != null)
				ParserNeutro.escribeInstruccionesXML(instrucciones, xsw);
			xsw.writeStartElement("choiceInteraction");		// Interaccin tipo choice
			xsw.writeAttribute("responseIdentifier","RESPONSE");
			xsw.writeAttribute("shuffle",Boolean.toString(shuffle));	// Mezclar respuestas
			xsw.writeAttribute("maxChoices",Integer.toString(maxChoices));// Mximas elecciones
			if (pregunta != null && !pregunta.equals("")){
				xsw.writeStartElement("prompt");			// Escribe la pregunta si la hay
				xsw.writeCharacters(pregunta);
				xsw.writeEndElement();						// Cierra "prompt"
			}
			for (int i = 0; i < respuestas.length; i++){	// Escribe las respuestas posibles
				xsw.writeStartElement("simpleChoice");
				xsw.writeAttribute("identifier", identificadores[i].getIdentificador());
				if (shuffle)								// Si mezcla marca las fijas
					if (fija[i])							// Respuesta fija
						xsw.writeAttribute("fixed", Boolean.toString(fija[i]));
				xsw.writeCharacters(respuestas[i]);			// Respuestas
				xsw.writeEndElement();						// Cierra "simpleChoice"
			}
			xsw.writeEndElement();							// Cierra "choiceInteraction"
			// Escribe el final del tem XML y cierra el writer
			writePie(xsw, sc, req);
			
			// Vaca y cierra el Writer
			sw.flush();
			sw.close();
			// Pasa a un string el contenido del Writer
			String fich = sw.toString();
			// Fichero de salida
			File file = new File(path + identificador + ".xml");
			// Escribe en el fichero de salida el contenido del tem
			FileUtils.writeStringToFile(file, fich, null);			
		} catch (XMLStreamException e) {
			// Error en el procesamiento de los datos XML
			msg.setMensaje("Error procesando el archivo XML");
			msg.setEstado(false);
		}
		if (msg.isEstado()){
			// Si todo ha ido bien se iforma del resultado
			msg.setMensaje("tem creado con xito");	
		}
		
		return msg;
	}
}