package xml.items;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import javax.servlet.ServletContext;

import org.apache.commons.io.FileUtils;

import utilidades.*;

/**
 * Clase contenedora del tem tipo "Gap Match". Contiene un campo para cada parmetro del tem 
 * necesario para construir el archivo del tem XML. Con los mtodos que incluye permite crear 
 * un tem tipo "Gap Match" completo y crear el archivo XML en disco.
 * 
 * @author 	David Domnguez
 * @see		Identificador
 * @see		EnteroPositivo
 */
public class GapmatchXML extends AssessmentItem {

	/**
	 * Pregunta del tem.
	 */
	protected String pregunta;
	/**
	 * Mezclar las opciones aleatoriamente o no.
	 */
	protected boolean shuffle;
	/**
	 * Textos en los que estn los huecos imbuidos.
	 */
	protected String[] textos;
	/**
	 * Valores posibles que pueden tomar los huecos.
	 */
	protected String[] opciones;
	/**
	 * Identificadores de las opciones.
	 */
	protected Identificador[] identOpciones;
	/**
	 * Identificadores de los huecos.
	 */
	protected Identificador[] identHuecos;
	/**
	 * Mximo de asociaciones de cada opcin.
	 */
	protected EnteroPositivo[] maxOpciones;
	/**
	 * Tabla con el nmero de la opcin correcta de cada hueco. Es el ndice de las tablas de 
	 * opciones.
	 */
	protected int[] respCorrecta;
	/**
	 * Array con las opciones que deben ser fijas.
	 */
	protected boolean[] fija;
	/**
	 * Cardinalidad de la variable respuesta.
	 */
	protected String cardinalidad;
	
	/**
	 * Constructor que permite inicializar el objeto con un primer grupo de parmetros. Son 
	 * los que se recogen en la primera pgina de la toma de datos. Para los parmetros 
	 * comunes a todos los tems se llama al constructor de la superclase.
	 * 
	 * @param 	identificador	identificador del tem
	 * @param 	titulo			ttulo del tem
	 * @param 	instrucciones	instrucciones del tem
	 * @param 	pregunta		pregunta del tem
	 * @param 	shuffle			mezclar las respuestas o no
	 */
	public GapmatchXML(String identificador, String titulo, String instrucciones, 
			String pregunta, boolean shuffle) {
		super(identificador, titulo, instrucciones);
		this.pregunta = pregunta;
		this.shuffle = shuffle;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de objetos <code>Identificador</code> que 
	 * contiene en cada posicin el identificador del hueco en esa misma posicin.
	 * 
	 * @param 	identHuecos		array con los identificadores de los huecos
	 */
	public void setIdentHuecos(Identificador[] identHuecos) {
		this.identHuecos = identHuecos;
		// Si el nmero de huecos es 1 la cardinalidad ser simple
		switch(identHuecos.length){
			case 1: cardinalidad = "single";break;
			default: cardinalidad = "multiple";break;
		}
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de objetos <code>Identificador</code> que 
	 * contiene en cada posicin el identificador de la opcin en esa misma posicin.
	 * 
	 * @param 	identOpciones		array con los identificadores de las opciones
	 */
	public void setIdentOpciones(Identificador[] identOpciones) {
		this.identOpciones = identOpciones;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>EnteroPositivo</code> conteniendo 
	 * en cada elemento el nmero mximo de huecos con los que esa opcin se puede asociar.
	 * 
	 * @param 	maxOpciones		array con los nmeros mximos de asociaciones de cada opcin
	 */
	public void setMaxOpciones(EnteroPositivo[] maxOpciones) {
		this.maxOpciones = maxOpciones;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>String</code> conteniendo en cada 
	 * elemento una de las opciones con las que se pueden asociar los huecos.
	 * 
	 * @param 	opciones		array con las opciones
	 */
	public void setOpciones(String[] opciones) {
		this.opciones = opciones;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>String</code> con los textos que 
	 * rodean a los huecos. Hay un texto ms que el nmero de huecos.
	 * 
	 * @param 	textos			array con los textos que rodean a los huecos
	 */
	public void setTextos(String[] textos) {
		this.textos = textos;
	}
		
	/**
	 * Mtodo que inicializa el <code>array</code> de enteros que indica la posicin de la 
	 * opcin que es correcta para cada hueco.
	 * 
	 * @param 	respCorrecta	array de enteros con la posicin de la opcin correcta de cada 
	 * 							hueco
	 */
	public void setRespCorrecta(String[] respCorrecta) {
		this.respCorrecta = new int[respCorrecta.length];
		for (int i = 0; i < respCorrecta.length; i++)
			this.respCorrecta[i] = Integer.parseInt(respCorrecta[i]);
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>boolean</code> indicando si esa 
	 * opcin debe quedarse en esa posicin o mezclarse si se deben mezclar aleatoriamente.
	 * 
	 * @param 	fija			array de boolean indicando posiciones fijas
	 */
	public void setFija(boolean[] fija) {
		this.fija = fija;
	}

	/**
	 * Mtodo que construye el tem XML tipo "Gap Match" con los campos que tiene almacenados 
	 * en la direccin que recibe como parmetro. Devuelve informacin de estado.
	 * 
	 * @param 	path		direccin absoluta en la que crear el archivo XML
	 * @param	sc			servletContext para acceder a los parmetros iniciales de "web.xml"
	 * @param	req			string con la url base de la Aplicacin Web requerida
	 * @return				informacin del estado final del proceso
	 * @throws 	IOException	si hay algn problema al intentar escribir el archivo en disco
	 * @see		XMLOutputFactory#createXMLStreamWriter(java.io.Writer)
	 * @see		XMLStreamWriter
	 * @see		StringWriter
	 */
	public MensajeEstado creaItemXML(String path, ServletContext sc, String req) 
			throws IOException {
		
		MensajeEstado msg = new MensajeEstado();		// Mensaje de salida del mtodo
		
		try {
			XMLOutputFactory xof =  XMLOutputFactory.newInstance();
			XMLStreamWriter xsw = null;				// Writer para escribir datos XML
			StringWriter sw = new StringWriter();	// Buffer en el que escribir
			xsw = xof.createXMLStreamWriter(sw);	// El writer XML escribe en el buffer
			// Escribe cabecera comn a todos los tipos de pregunta
			writeCabecera(xsw, sc);
			xsw.writeAttribute("identifier", identificador);
			xsw.writeAttribute("title", titulo);
			xsw.writeStartElement("responseDeclaration");	// Declara la variable de respuesta
			xsw.writeAttribute("identifier", "RESPONSE");
			xsw.writeAttribute("cardinality", cardinalidad);
			xsw.writeAttribute("baseType", "identifier");
			xsw.writeStartElement("correctResponse");	// Valores de las respuestas correctas
			for (int i = 0; i < identHuecos.length; i++){
				xsw.writeStartElement("value");			// Valor correcto
				xsw.writeCharacters(identHuecos[i].getIdentificador() + " ");
				xsw.writeCharacters(identOpciones[respCorrecta[i]].getIdentificador());
				xsw.writeEndElement();					// Cierra "value"
			}
			xsw.writeEndElement();							// Cierra "correctResponse"
			xsw.writeEndElement();							// Cierra "responseDeclaration"
			xsw.writeStartElement("outcomeDeclaration");	// Variable de puntuacin
			xsw.writeAttribute("identifier", "SCORE");
			xsw.writeAttribute("cardinality", "single");
			xsw.writeAttribute("baseType", "integer");
			xsw.writeStartElement("defaultValue");			// Valor por defecto de la salida
			xsw.writeStartElement("value");					// Valor
			xsw.writeCharacters("0");
			xsw.writeEndElement();							// Cierra "value"
			xsw.writeEndElement();							// Cierra "defaultValue"
			xsw.writeEndElement();							// Cierra "outcomeDeclaration"
			xsw.writeStartElement("itemBody");				// Cuerpo del tem
			// Las instrucciones se escriben mediante el parser neutro puesto que es xhtml, sin 
			// que el writer XML modifique los datos
			if (instrucciones != null)
				ParserNeutro.escribeInstruccionesXML(instrucciones, xsw);
			xsw.writeStartElement("gapMatchInteraction");	// Interaccin tipo GapMatch
			xsw.writeAttribute("responseIdentifier","RESPONSE");
			xsw.writeAttribute("shuffle",Boolean.toString(shuffle));		// Mezclar opciones
			if (pregunta != null && !pregunta.equals("")){
				xsw.writeStartElement("prompt");			// Escribe la pregunta si la hay
				xsw.writeCharacters(pregunta);
				xsw.writeEndElement();						// Cierra "prompt"
			}
			for (int i = 0; i < identOpciones.length; i++){	// Escribe las opciones posibles
				xsw.writeStartElement("gapText");
				xsw.writeAttribute("identifier", identOpciones[i].getIdentificador());
				xsw.writeAttribute("matchMax",Integer.toString(
						maxOpciones[i].getEnteroPositivo()));
				if (shuffle)								// Si mezcla marca las fijas
					if (fija[i])							// Respuesta fija
						xsw.writeAttribute("fixed", Boolean.toString(fija[i]));
				xsw.writeCharacters(opciones[i]);
				xsw.writeEndElement();						// Cierra "gapText"
			}
			xsw.writeStartElement("blockquote");
			xsw.writeStartElement("p");						// Mete el texto en un prrafo
			for (int i = 0; i < identHuecos.length; i++){
				if (textos[i] != null && !textos[i].equals(""))			// Escribe texto 'i'
					// Los Textos se escriben mediante el mtodo de escribir textos XML para 
					// sustituir los caracteres de nueva lnea por los equivalentes <br /> de 
					// XHTML
					ParserNeutro.escribeTextoXML(textos[i], xsw);
				xsw.writeEmptyElement("gap");				// Hueco 'i'
				// Identificador del hueco 'i'
				xsw.writeAttribute("identifier", identHuecos[i].getIdentificador());
			}
			// Escribe el ltimo texto
			if (textos[textos.length - 1] != null && !textos[textos.length - 1].equals(""))
				ParserNeutro.escribeTextoXML(textos[textos.length - 1], xsw);
			xsw.writeEndElement();							// Cierra "p"
			xsw.writeEndElement();							// Cierra "blockquote"
			xsw.writeEndElement();							// Cierra "gapMatchInteraction"
			// Escribe el final del tem XML y cierra el writer
			writePie(xsw, sc, req);
			
			// Vaca y cierra el Writer
			sw.flush();
			sw.close();
			// Escribe en un String el contenido del Writer
			String fich = sw.toString();
			// Fichero de salida
			File file = new File(path + identificador + ".xml");
			// Escribe en el fichero de salida
			FileUtils.writeStringToFile(file, fich, null);			
		} catch (XMLStreamException e) {
			// Error en el procesamiento de los datos XML
			msg.setMensaje("Error procesando el archivo XML");
			msg.setEstado(false);
		}
		if (msg.isEstado()){
			// Si todo ha ido bien se iforma del resultado
			msg.setMensaje("tem creado con xito");	
		}
		
		return msg;
	}
}