package xml.items;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import javax.servlet.ServletContext;

import org.apache.commons.io.FileUtils;

import utilidades.*;

/**
 * Clase contenedora del tem tipo "Hot Text". Contiene un campo para cada parmetro del tem 
 * necesario para construir el archivo del tem XML. Con los mtodos que incluye permite crear 
 * un tem tipo "Hot Text" completo y crear el archivo XML en disco.
 * 
 * @author 	David Domnguez
 * @see		Identificador
 */
public class HottextXML extends AssessmentItem {

	/**
	 * Pregunta del tem.
	 */
	protected String pregunta;
	/**
	 * Textos que rodean a los hot texts.
	 */
	protected String[] textos;
	/**
	 * Hot texts a incluir en el texto.
	 */
	protected String[] hottexts;
	/**
	 * Identificadores de los hottexts.
	 */
	protected Identificador[] identHottexts;
	/**
	 * Mximo nmero de hot texts seleccionables.
	 */
	protected int maxChoices;
	/**
	 * Array de boolean con los hottexts correctos.
	 */
	protected boolean[] correcto;
	/**
	 * Cardinalidad de la variable de respuesta.
	 */
	protected String cardinalidad;

	/**
	 * Constructor que permite inicializar el objeto con un primer grupo de parmetros. Son 
	 * los que se recogen en la primera pgina de la toma de datos. Para los parmetros 
	 * comunes a todos los tems se llama al constructor de la superclase.
	 * 
	 * @param 	identificador	identificador del tem
	 * @param 	titulo			ttulo del tem
	 * @param 	instrucciones	instrucciones del tem
	 * @param 	pregunta		pregunta del tem
	 */
	public HottextXML(String identificador, String titulo, String instrucciones, 
			String pregunta) {
		super(identificador, titulo, instrucciones);
		this.pregunta = pregunta;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>boolean</code> que indica si el 
	 * hot text en esa posicin es correcto o no.
	 * 
	 * @param 	correcto		array de boolean indicando hot text correcto
	 */
	public void setCorrecto(boolean[] correcto) {
		this.correcto = correcto;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>String</code> que contiene cada 
	 * uno de los hot texts a incluir en el texto.
	 * 
	 * @param 	hottexts 		array con los hot texts
	 */
	public void setHottexts(String[] hottexts) {
		this.hottexts = hottexts;
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de objetos <code>Identificador</code> que 
	 * contiene en cada posicin el identificador del hot text en esa misma posicin.
	 * 
	 * @param 	identHottexts	array con los identificadores de los hot texts
	 */
	public void setIdentHottexts(Identificador[] identHottexts) {
		this.identHottexts = identHottexts;
	}

	/**
	 * Mtodo que inicializa el nmero mximo de hot texts que se pueden seleccionar como 
	 * correctos. En funcin de ste, inicializa <code>cardinalidad</code> a un valor adecuado.
	 * 
	 * @param 	maxChoices		entero indicando el nmero mximo de elecciones
	 */
	public void setMaxChoices(int maxChoices) {
		this.maxChoices = maxChoices;
		switch(maxChoices){
			case 1: cardinalidad = "single"; break;
			default: cardinalidad = "multiple"; break;
		}
	}

	/**
	 * Mtodo que inicializa el <code>array</code> de <code>String</code> con los textos que 
	 * rodean a los huecos. El nmero de textos es igual al nmero de huecos ms uno.
	 * 
	 * @param 	textos		array con los textos que rodean a los huecos
	 */
	public void setTextos(String[] textos) {
		this.textos = textos;
	}

	/**
	 * Mtodo que construye el tem XML tipo "Hot Text" con los campos que tiene almacenados 
	 * en la direccin que recibe como parmetro. Devuelve informacin de estado.
	 * 
	 * @param 	path		direccin absoluta en la que crear el archivo XML	
	 * @param	sc			servletContext para acceder a los parmetros iniciales de "web.xml"
	 * @param	req			string con la url base de la Aplicacin Web requerida
	 * @return				informacin del estado final del proceso
	 * @throws 	IOException	si hay algn problema al intentar escribir el archivo en disco
	 * @see		XMLOutputFactory#createXMLStreamWriter(java.io.Writer)
	 * @see		XMLStreamWriter
	 * @see		StringWriter
	 */
	public MensajeEstado creaItemXML(String path, ServletContext sc, String req) 
			throws IOException {
		
		MensajeEstado msg = new MensajeEstado();		// Mensaje de salida del mtodo
		
		try {
			XMLOutputFactory xof =  XMLOutputFactory.newInstance();
			XMLStreamWriter xsw = null;				// Writer para escribir datos XML
			StringWriter sw = new StringWriter();	// Buffer en el que escribir
			xsw = xof.createXMLStreamWriter(sw);	// El writer XML escribe en el buffer
			// Escribe cabecera comn a todos los tipos de pregunta
			writeCabecera(xsw, sc);
			xsw.writeAttribute("identifier", identificador);
			xsw.writeAttribute("title", titulo);
			xsw.writeStartElement("responseDeclaration");	// Declara la variable de respuesta
			xsw.writeAttribute("identifier", "RESPONSE");
			xsw.writeAttribute("cardinality", cardinalidad);
			xsw.writeAttribute("baseType", "identifier");
			xsw.writeStartElement("correctResponse");		// Respuestas correctas
			for (int i = 0; i < correcto.length; i++){
				if (correcto[i]){
					xsw.writeStartElement("value");			// Valor correcto
					xsw.writeCharacters(identHottexts[i].getIdentificador());
					xsw.writeEndElement();					// Cierra "value"	
				}
			}
			xsw.writeEndElement();							// Cierra "correctResponse"
			xsw.writeEndElement();							// Cierra "responseDeclaration"
			xsw.writeStartElement("outcomeDeclaration");	// Variable de puntuacin
			xsw.writeAttribute("identifier", "SCORE");
			xsw.writeAttribute("cardinality", "single");
			xsw.writeAttribute("baseType", "integer");
			xsw.writeStartElement("defaultValue");			// Valor por defecto de la salida
			xsw.writeStartElement("value");					// Valor
			xsw.writeCharacters("0");
			xsw.writeEndElement();							// Cierra "value"
			xsw.writeEndElement();							// Cierra "defaultValue"
			xsw.writeEndElement();							// Cierra "outcomeDeclaration"
			xsw.writeStartElement("itemBody");				// Cuerpo del tem
			// Las instrucciones se escriben directamente puesto que es xhtml, sin que el 
			// writer XML modifique los datos
			if (instrucciones != null)
				ParserNeutro.escribeInstruccionesXML(instrucciones, xsw);
			xsw.writeStartElement("hottextInteraction");	// Interaccin tipo hottext
			xsw.writeAttribute("responseIdentifier","RESPONSE");
			xsw.writeAttribute("maxChoices",Integer.toString(maxChoices));
			if (pregunta != null && !pregunta.equals("")){
				xsw.writeStartElement("prompt");			// Escribe la pregunta si la hay
				xsw.writeCharacters(pregunta);
				xsw.writeEndElement();						// Cierra "prompt"
			}
			xsw.writeStartElement("p");						// Mete el texto en un prrafo
			for (int i = 0; i < identHottexts.length; i++){
				if (textos[i] != null && !textos[i].equals(""))			// Escribe texto 'i'
					// Los Textos se escriben mediante el mtodo de escribir textos XML para 
					// sustituir los caracteres de nueva lnea por los equivalentes <br /> de 
					// XHTML
					ParserNeutro.escribeTextoXML(textos[i], xsw);
				xsw.writeStartElement("hottext");			// Hottext 'i'
				xsw.writeAttribute("identifier", identHottexts[i].getIdentificador());
				xsw.writeCharacters(hottexts[i]);
				xsw.writeEndElement();						// Cierra "hottext" 'i'
			}
			// Escribe el ltimo texto
			if (textos[textos.length - 1] != null && !textos[textos.length - 1].equals(""))
				ParserNeutro.escribeTextoXML(textos[textos.length - 1], xsw);
			xsw.writeEndElement();							// Cierra "p"
			xsw.writeEndElement();							// Cierra "hottextInteraction"
			// Escribe el final del tem XML y cierra el writer
			writePie(xsw, sc, req);
			
			// Vaca y cierra el Writer
			sw.flush();
			sw.close();
			// Escribe en un String el contenido del Writer
			String fich = sw.toString();
			// Fichero de salida
			File file = new File(path + identificador + ".xml");
			// Escribe en el fichero de salida
			FileUtils.writeStringToFile(file, fich, null);			
		} catch (XMLStreamException e) {
			// Error en el procesamiento de los datos XML
			msg.setMensaje("Error procesando el archivo XML");
			msg.setEstado(false);
		}
		if (msg.isEstado()){
			// Si todo ha ido bien se iforma del resultado
			msg.setMensaje("tem creado con xito");	
		}
		
		return msg;
	}
}