package xml.test;

import java.io.File;
import java.net.URI;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import utilidades.Identificador;

/**
 * Clase que representa las referencias a los tems individuales dentro de un Test segn la 
 * norma QTI del IMS. Se utiliza para incorporar cada una de las cuestiones individuales dentro 
 * de un Test. Contiene varios atributos que se corresponden con los atributos definidos en la 
 * norma, ms una variable tipo <code>File</code> del archivo del tem a incluir en el test 
 * para ayudar a calcular su URI relativa. Tal y como define la norma, esta clase hereda de la 
 * clase "SectionPart", por lo tanto tambin contendr sus variables de clase.
 * 
 * @author David Domnguez
 *
 */
public class AssessmentItemRef extends SectionPart {


	/**
	 * Objeto <code>File</code> apuntando al tem XML al que hace referencia esta clase.
	 */
	private File itemFile;

	/**
	 * URI relativa desde el test con la referencia al tem XML que se debe incluir en el Test.
	 */
	private URI href;
	/**
	 * Objeto de tipo <code>Weight</code> con el peso que aplicar a la pregunta en caso de que 
	 * sea correcta
	 */
	private Weight ok;
	/**
	 * Objeto de tipo <code>Weight</code> con el peso que aplicar a la pregunta en caso de que 
	 * sea incorrecta
	 */
	private Weight noOk;
	
	/**
	 * Constructor de la clase referencia a un tem. Le da los valores iniciales a los pesos 
	 * correcto e incorrecto del tem, al objeto <code>File</code> que apunta al tem, y llama 
	 * al constructor de la superclase para inicializar sus variables de clase con el resto de 
	 * los parmetros.
	 * 
	 * @param 	identificador	identificador de la referencia del tem
	 * @param 	requerido		boolean indicando si el tem es requerido
	 * @param 	fijo			boolean indicando si el tem est en posicin fija
	 * @param 	itemFile		file del tem al que hace referencia la clase
	 * @param 	ok				weight con el peso del tem en caso de respuesta correcta
	 * @param 	noOk			weight con el peso del tem en caso de respuesta incorrecta
	 */
	public AssessmentItemRef(Identificador identificador, boolean requerido, boolean fijo, 
			File itemFile, Weight ok, Weight noOk) {
		super(identificador, requerido, fijo);
		this.itemFile = itemFile;
		this.ok = ok;
		this.noOk = noOk;
	}

	/**
	 * Calcula la URI de referencia del tem a partir de la localizacin del propio tem y de 
	 * la del test que se est creando, hallando la direccin relativa del tem respecto al test.
	 * 
	 * @param 	test	file del test que se est creando
	 * @see		java.net.URI
	 */
	public void creaItemRef (File test){
		// URI absoluta del tem
		String itemAbsoluta = itemFile.toURI().toString();
		// URI absoluta del test
		String  testPath = test.toURI().toString();
		// String para calcular la parte de la direccin que comparten el test y el tem
		String comun;
		// ndice para ir avanzando por cada uno de los separadores de la direccin del tem
		int indexAbs = 0;
		// Boolean indicando que ya se ha encontrado la direccin comn entre el test y el tem
		boolean fin;
		// Va avanzando por los separadores hasta que sean diferentes las direcciones del test 
		// y del tem
		do {
			// localizacin del carcter separador desde la posicin anterior
			indexAbs = itemAbsoluta.indexOf('/', indexAbs + 1);
			// Direccin base comn a probar si es comn de ambas
			comun = itemAbsoluta.substring(0, indexAbs);
			// Se comprueba si esta direccin es comn a ambas direcciones
			fin = testPath.startsWith(comun);
		} while (fin == true);
		// La parte comn de la direccin es desde el comienzo hasta el ltimo carcter 
		// separador
		comun = comun.substring(0, comun.lastIndexOf('/'));
		// Direccin absoluta del tem menos la parte comn con la direccin absoluta del test 
		String rel = itemAbsoluta.substring(comun.length() + 1, itemAbsoluta.length());
		// Direccin absoluta del test menos la parte comn con la direccin absoluta del tem
		testPath = testPath.substring(comun.length() + 1, testPath.length());
		// Clculo del nmero de separadores que hay desde la localizacin del test hasta 
		// llegar a la direccin comn con el tem. Es el nmero de saltos de directorios 
		// "hacia arriba" que hay que hacer para llegar al tem desde el test
		int numDir = 0;
		for (int i = 0; i < testPath.length(); i++){
			if (testPath.charAt(i) == '/')
				numDir++;
		}
		// Direccin relativa del tem respecto al test
		String relativa = "";
		// Primero se incluyen los caracteres de subir nivel en la estructura de directorios 
		// para llegar a la parte comn la direccin absoluta del tem
		for (int i = 0; i < numDir; i++)
			relativa += "../";
		// Luego se aade la direccin del tem menos la parte que tiene en comn con el test
		relativa += rel;
		// Se crea una URI relativa a partir de la direccin relativa contenida en el String
		href = URI.create(relativa);
	}
	
	/**
	 * Mtodo que escribe este objeto <code>AssessmentItemRef</code> en el <code>XMLStreamWriter
	 * </code> recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaAssessmentItemRef (XMLStreamWriter xsw) throws XMLStreamException {
		
		xsw.writeStartElement("assessmentItemRef");		// Comienzo del elemento
		// Identificador de la referencia al tem
		xsw.writeAttribute("identifier", identificador.getIdentificador());
		xsw.writeAttribute("href", href.toString());	// URI de referencia al tem
		if (requerido == true)	// El elemento requerido se escribe si el tem es requerido
			xsw.writeAttribute("required", Boolean.toString(requerido));
		if (fijo == true)		// El elemento fijo se escribe si el tem se queda fijo
			xsw.writeAttribute("fixed", Boolean.toString(fijo));
		ok.creaWeight(xsw);	   // Se llama al mtodo que escriba el peso de respuesta correcta
		noOk.creaWeight(xsw);  // Se llama al mtodo que escriba el peso de respuesta incorrecta
		xsw.writeEndElement(); // Cierra el elemento "assessmentItemRef"
	}
}