package xml.test;

import java.io.File;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import utilidades.Identificador;
import utilidades.ParserNeutro;

/**
 * Clase que representa a una seccin del Test segn la norma QTI del IMS. Se utiliza para 
 * agrupar varios tems dentro de un Test. Contiene varios atributos que se corresponden con 
 * los atributos definidos en la norma, como son el ttulo de la seccin, si es visible, si se 
 * sigue algn criterio de seleccin y ordenacin, y el array de objetos <code>
 * AssessmentItemRef</code> con las referencias de los tems incluidos en la seccin. Tal y 
 * como define la norma esta clase hereda de la clase "SectionPart", por lo tanto tambin 
 * contendr sus variables de clase.

 * @author David Domnguez
 *
 */
public class AssessmentSection extends SectionPart {

	/**
	 * Ttulo de la seccin.
	 */
	private String titulo;
	/**
	 * Si la seccin es visible para el candidato que realice el test.
	 */
	private boolean visible;
	/**
	 * Criterio de seleccin de los tems posibles a seguir.
	 */
	private Selection seleccion;
	/**
	 * Criterio de ordenacin de los tems del test.
	 */
	private Ordering orden;
	/**
	 * Instrucciones del tem. Es cdigo XHTML.
	 */
	private String instrucciones;
	/**
	 * Array de referencias a los tems que se incluyen en la seccin.
	 */
	private AssessmentItemRef[] itemRefArray;
	
	/**
	 * Constructor que inicializa variables de clase. Inicializa las siguientes variables de 
	 * clase: <code>titulo</code>, <code>visible</code>, <code>seleccion</code>, <code>orden
	 * </code>, e <code>instrucciones</code>, y llamando al constructor de la superclase con el 
	 * identificador e iniciando ambos atributos de seccin requerida y fija a falso, ya que 
	 * esta implementacin de la norma QTI del IMS slo permite una seccin dentro del test, 
	 * por lo que esos parmetros no tendrn en cuenta.
	 * 
	 * @param 	identificador		identificador de la seccin
	 * @param 	titulo				string con el ttulo de la seccin
	 * @param 	visible				boolean indicando la visibilidad de la seccin
	 * @param 	seleccion			selection con el criterio de seleccin de los tems
	 * @param 	orden				ordering con el criterio de ordenacin de los tems
	 * @param 	instrucciones		string con las instrucciones XHTML
	 */
	public AssessmentSection(Identificador identificador, String titulo, boolean visible, 
			Selection seleccion, Ordering orden, String instrucciones) {
		// Los parmetros requerido y fijo para la seccin, como slo hay una, no son 
		// necesarios y no se tienen en cuenta
		super(identificador, false, false);
		this.titulo = titulo;
		this.visible = visible;
		this.seleccion = seleccion;
		this.orden = orden;
		this.instrucciones = instrucciones;
	}

	/**
	 * Mtodo "get" para obtener el array de objetos de referencia a los tems, <code>
	 * AssessmentItemRef</code>, de la seccin.
	 * 
	 * @return		array de objetos AssessmentItemRef de la seccin
	 * @see 		#setItemRefArray
	 */
	public AssessmentItemRef[] getItemRefArray() {
		return itemRefArray;
	}

	/**
	 * Mtodo "set" para establecer el array de objetos de referencia a los tems,<code>
	 * AssessmentItemRef</code>, de la seccin.
	 *  
	 * @param 	itemRefArray	array de objetos AssessmentItemRef de la seccin
	 * @see #getItemRefArray
	 */
	public void setItemRefArray(AssessmentItemRef[] itemRefArray) {
		this.itemRefArray = itemRefArray;
	}

	/**
	 * Mtodo "get" para obtener el criterio de ordenacin de la seccin.
	 * 
	 * @return		ordering con el criterio de ordenacin
	 */
	public Ordering getOrden() {
		return orden;
	}

	/**
	 * Mtodo "get" para obtener el criterio de seleccin de los tems de la seccin.
	 * 
	 * @return		selection con el criterio de seleccin
	 */
	public Selection getSeleccion() {
		return seleccion;
	}
	
	/**
	 * Calcula la URI relativa de todos los tems de la seccin. El mtodo llama al mtodo para 
	 * calcular la URI relativa de un tem para cada uno de los objetos referencia de tem de 
	 * la seccin, <code>AssessmentItemRef</code>, pasndole el objeto <code>File</code> 
	 * representando la direccin en disco del Test.
	 * 
	 * @param 	referencia		file con la direccin final en disco del Test
	 */
	public void creaRefItems (File referencia){
		for (int i = 0; i < itemRefArray.length; i++)
			itemRefArray[i].creaItemRef(referencia);
	}
	
	/**
	 * Mtodo que escribe este objeto <code>AssessmentSection</code> en el <code>XMLStreamWriter
	 * </code> recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaAssessmentSection (XMLStreamWriter xsw) throws XMLStreamException{
		
		xsw.writeStartElement("assessmentSection");		// Comienzo del elemento
		xsw.writeAttribute("identifier", identificador.getIdentificador());	// Identificador
		xsw.writeAttribute("title", titulo);			// Ttulo de la seccin
		xsw.writeAttribute("visible", Boolean.toString(visible));	// Atributo visible
		if (seleccion != null)							// Escribe la seleccin si la hay
			seleccion.creaSelection(xsw);
		if (orden != null)								// Escribe el ordenacin si lo hay
			orden.creaOrdering(xsw);
		if (instrucciones != null) {					// Las instrucciones se escriben si hay
			xsw.writeStartElement("rubricBlock");		// Elemento para las instrucciones
			xsw.writeAttribute("view", "candidate");	// Las ve el candidato
			// Se escriben mediante el parser neutro ya que es cdigo XHTML
			ParserNeutro.escribeInstruccionesXML(instrucciones, xsw);
			xsw.writeEndElement();						// Cierra "rubricBlock"
		}
		for (int i = 0; i < itemRefArray.length; i++)
			// Escribe cada una de las referencias a los tems
			itemRefArray[i].creaAssessmentItemRef(xsw);
		xsw.writeEndElement();							// Cierra "assessmentSection"
	}
}