package xml.test;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import utilidades.Identificador;

/**
 * Clase que representa una Parte de un Test segn la norma QTI del IMS. Se utiliza para 
 * agrupar Secciones. Contiene unos atributos que se corresponden con los definidos en la norma.
 * 
 * @author David Domnguez
 *
 */
public class TestPart {

	/**
	 * Identificador de la Parte del Test.
	 */
	private Identificador identificador;
	/**
	 * Modo de navegacin por la Parte del Test.
	 */
	private String navegacion;
	/**
	 * Modo de presentacin de la Parte del Test.
	 */
	private String presentacion;
	/**
	 * Objeto de control de la Parte del Test.
	 */
	private ItemSessionControl control;
	/**
	 * Seccin que incluye esta Parte del Test. Esta implementacin slo soporta una Seccin por 
	 * Parte de Test.
	 */
	private AssessmentSection seccion;
	
	/**
	 * Constructor que simplemente inicializa las variables de la clase.
	 * 
	 * @param 	identificador	identificador con un identificador vlido
	 * @param 	navegacion		string con el modo de navegacin
	 * @param 	presentacion	string con el modo de presentacin
	 * @param 	control			itemSessionControl con el control de la sesin
	 * @param 	seccion			assessmentSection con la seccin que incluye
	 */
	public TestPart(Identificador identificador, String navegacion, String presentacion, 
			ItemSessionControl control, AssessmentSection seccion) {
		this.identificador = identificador;
		this.navegacion = navegacion;
		this.presentacion = presentacion;
		this.control = control;
		this.seccion = seccion;
	}
	
	/**
	 * Mtodo "get" para obtener la Seccin, el objeto <code>AssessmentSection</code> que 
	 * incluye esta Parte del Test.
	 * 
	 * @return					assessmentSection con la Seccin
	 */
	public AssessmentSection getSeccion() {
		return seccion;
	}
	
	/**
	 * Mtodo que escribe este objeto <code>TestPart</code> en el <code>XMLStreamWriter</code> 
	 * recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaTestPart(XMLStreamWriter xsw) throws XMLStreamException{
		xsw.writeStartElement("testPart");					// Escribe el elemento
		xsw.writeAttribute("identifier", identificador.getIdentificador());	// Identificador
		xsw.writeAttribute("navigationMode", navegacion);	// Modo de navegacin
		xsw.writeAttribute("submissionMode", presentacion);	// Modo de presentacin
		control.creaItemSessionControl(xsw);				// Escribe el control de la sesin
		seccion.creaAssessmentSection(xsw);					// Escribe la seccin
		xsw.writeEndElement();								// Cierra "testPart"
	}
}